
from setrun import setrun
from setplot import setplot
from pyclaw.runclaw import runclaw
from pyclaw.plotters.plotclaw import plotclaw

from numpy import *
import pylab
import os

# initialize rundata using setrun but then change some things for each run:
rundata = setrun()
rundata.clawdata.dt_variable = 0
rundata.clawdata.nout = 1


rundata.clawdata.xlower =  0.25
rundata.clawdata.xupper =  1.
rundata.clawdata.ylower = .25
rundata.clawdata.yupper =  1.

mx_base = 10
my_base = 10

t_final_target = 1.

rundata.probdata.nthreads = 3


#===> Solve for each dt ===>

def refinementTest( dt_values ):
    
    n_runs = len(dt_values)

    
    error_values = zeros(n_runs)
    

    

    for i_run in range(n_runs):
        
        
        #---- Set grid size ----
        refinement_factor   = dt_values[0] / dt_values[i_run]
        rundata.clawdata.mx = 200 #int( mx_base * refinement_factor )
        rundata.clawdata.my = 200 #int( my_base * refinement_factor )

        #---- Discretization parameters for this run ----
        dt      = dt_values[i_run]
        t_final = round(t_final_target/dt)*dt
    
        #---- Write parameters to clawdata ----
        rundata.clawdata.dt_initial = dt
        rundata.clawdata.tfinal     = t_final
        rundata.write()

        #---- Execute the run ----
        runclaw(xclawcmd = "xclaw", outdir="_output")
    
        #---- Read the final numerical error ----
        try:
            f = open("_output/error_value.txt", "r")
        except:
            print "Could not open _output/error_value.txt for dt = %f" % (dt)
            error_values[i_run] = nan
        else:
            error_values[i_run] = float(f.readline())
            f.close()
            os.system("rm _output/error_value.txt")
    
    return error_values
    
#<=== Solve for each dt <===


dt_values = .1 * logspace(0,-2,21)

error_values = refinementTest( dt_values )


#===> Produce error plots ===>


#---- Plot true solution ----
plot_parameters = {'backend': 'eps',
          'legend.borderpad':    0.4,
          'legend.labelspacing': 0.4,
          'text.fontsize': 16,
          'font.size'    : 12,
          'legend.fontsize': 12,
          'xtick.labelsize': 12,
          'ytick.labelsize': 12,
          'axes.labelsize':  16,
          'figure.subplot.left':   0.15,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.18,
          'figure.subplot.top':    0.95}

pylab.rcParams.update(plot_parameters)

pylab.close('all')
pylab.figure(figsize=(4,3), facecolor='w', edgecolor='k')


pylab.loglog(dt_values, error_values, 'ko', label="Error values")

pylab.loglog(dt_values, dt_values,    'k-', label="$\Delta t$", linewidth=3)

# pylab.title('Error values for porous medium equation')
pylab.legend(loc='upper left')
pylab.xlim(1e-3/2, 2e-1)
pylab.ylim(1e-3/2, 1)
pylab.xlabel('$\Delta t$')

pylab.savefig('/Users/onath/Documents/Projects/Surfactant/surfactant_paper/paper/figures/AxisymmetricAccuracy.eps')

#<=== Produce error plots <===