subroutine src2 ( maxmx, maxmy, meqn, mbc, mx, my, x_low, y_low, dx, dy, q, maux, aux, t, dt )

!-----------------------------------------------------------------
! Controls integration of the parabolic terms.  The main content
! of this routine is the adaptive time stepping, which halves the
! time step for a parabolic integration routine in the event that
! Newton's method fails to converge.
!-----------------------------------------------------------------

    implicit none


    !===> Variables ===>

    !---- Globals ----
    double precision :: beta, kappa, mu, delta
    common /param_block/    beta, kappa, mu, delta

    !---- Inputs ----
    integer, intent(in) :: maxmx, maxmy, meqn, mbc, mx, my, maux
    double precision, intent(in) :: x_low, y_low, dx, dy, t, dt
    double precision, intent(in) :: aux(1-mbc:maxmx+mbc, 1-mbc:maxmy+mbc, maux)

    !---- In/out ----
    double precision, intent(inout) :: q(1-mbc:maxmx+mbc, 1-mbc:maxmy+mbc, meqn)

    !---- Locals ----    
    integer :: ix, iy, max_splits, n_splits, step
    double precision :: tolerance, t_local, dt_local, t_old, t_new, max_val
    logical :: success

    !---- External ----
    double precision, external :: film_true

    !<=== Variables <===



    t_old = t-dt
    t_new = t






    !---- Set tolerance for Newton's method ----
    !------------------------------------------------------------
    ! I'd like to set this to dt**2, but that causes problems if
    ! the value is too small.
    !------------------------------------------------------------
    tolerance = max(dx**2, dy**2)
    print *, "Tolerance = ", tolerance

    max_splits = 8





    !---- Fill the film's ghost cells ----
    
    call fill_ghost_cells( q(:,:,1), film_true, t_new, x_low, y_low, dx, dy, mx, my, mbc )


    !---- Surfactant step ----
    t_local  = t_old
    dt_local = dt
    n_splits = 0
    success  = .false.
    

    do while ( t_new - t_local > 1d-8)
                        
        call surfactant_parabolic_step( q(:,:,2), q(:,:,1), aux(:,:,1), aux(:,:,2), tolerance, success, &
                                        delta, mu, t_local, dt_local,                                   &
                                        x_low, y_low, dx, dy, mx, my, mbc )
    
    
        if (success) then
            t_local  = t_local + dt_local
        else
            dt_local = dt_local / 2.d0
            n_splits = n_splits + 1
            print *, 'Splitting time step (surfactant)'
    
            if (n_splits > max_splits) then
                print '(A)', 'Surfactant parabolic step failed with maximum number of dt splits; aborting.'
                stop
            end if
        end if
    end do
    
    

    call write_numerical_error( q(:,:,1), film_true, t_new, x_low, y_low, dx, dy, mx, my, mbc )


    return
end subroutine src2
