subroutine surfactant_integrate_slice( rhs, g, h, delta, mu, dx, dt, mx, mbc )

!---------------------------------------------------------
! Solves a 1d problem for the ADI split on the surfactant
! equation.
!---------------------------------------------------------


    implicit none


    !===> Variables ===>

    !---- Inputs ----
    integer, intent(in) :: mx, mbc
    double precision, intent(in) :: g(1-mbc:mx+mbc), h(1-mbc:mx+mbc)
    double precision, intent(in) :: delta, mu, dx, dt

    !---- In/out ----
    double precision, intent(inout) :: rhs(mx)

    !---- Locals ----
    integer :: ipiv(mx), info, i, j, n_lower, n_upper
    parameter (n_lower=1, n_upper=1)
    double precision :: backward_euler(2*n_lower+1+n_upper, mx), sparse_flux(n_lower+n_upper,mx)

    !<=== Variables <===



    !---- Set sparse_flux ----
    call surfactant_sparse_flux( sparse_flux, g, h, delta, mu, dx, dt, mx, mbc )





    !---- Define backward_euler ----
    !--------------------------------------------------------------------------------
    ! The body of the loop implements
    !
    !       backward_euler(2,j) =        dt/dx *  sparse_flux(1,j)
    !       backward_euler(3,j) = 1.d0 + dt/dx * (sparse_flux(2,j) - sparse_flux(1,j))
    !       backward_euler(4,j) =        dt/dx * (                 - sparse_flux(2,j))
    !
    ! in a form that can be used with problems of other orders as well.
    !--------------------------------------------------------------------------------

    do j = 1, mx

        backward_euler(n_lower+1,j) = dt/dx * sparse_flux(1,j)
    
    
        do i = 2, n_lower+n_upper
            backward_euler(n_lower+i,j) = dt/dx * (sparse_flux(i,j) - sparse_flux(i-1,j))
        end do
    
        backward_euler(2*n_lower+1,j) = backward_euler(2*n_lower+1,j) + 1.d0
    
    
        backward_euler(2*n_lower+n_upper+1,j) = dt/dx * ( -sparse_flux(n_lower+n_upper,j) )

    end do
    
    



    !---- Solve; rhs is overwritten with the result ----

    call dgbsv(mx, n_lower, n_upper, 1, backward_euler, 2*n_lower+n_upper+1, ipiv, rhs, mx, info)


end subroutine surfactant_integrate_slice