from numpy import *
import pylab

from ClawSolution2D import ClawSolution2D


t0 = 4.1

def filmTrue( r, t ):
    tau = t0 + t
    return 2. * (r / tau**.25)**2
    

def surfactantTrue( r, t ):
    tau = t0 + t
    return -1./(8.*tau**.5) * log(r/tau**.25)



claw_sol = ClawSolution2D('_output', 1)



#---- Create flattened radial coordinate ----
x,y = claw_sol.cell_centers()
r = sqrt( x**2 + y**2 )
r = r.flatten()


#---- Extract flattened solution data ----
film_flat       = claw_sol.q[:,:,0].flatten()
surfactant_flat = claw_sol.q[:,:,1].flatten()


#---- Create true solution data on even grid ----
r_linear        = linspace(r.min(), r.max(), 101)
film_true       = filmTrue( r_linear, claw_sol.t )
surfactant_true = surfactantTrue( r_linear, claw_sol.t )



#---- Plot true solution ----
plot_parameters = {'backend': 'eps',
          'legend.borderpad':    0.8,
          'legend.labelspacing': 0.8,
          'text.fontsize': 12,
          'font.size'    : 12,
          'legend.fontsize': 12,
          'xtick.labelsize': 12,
          'ytick.labelsize': 12,
          'axes.labelsize':  12,
          'figure.subplot.left':   0.15,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.15,
          'figure.subplot.top':    0.9}

pylab.rcParams.update(plot_parameters)

pylab.close('all')

pylab.figure(figsize=(4,3), facecolor='w', edgecolor='k')
pylab.plot(r_linear, film_true, color='0.0', linewidth=2, label='Film', zorder=1)
pylab.scatter(r, film_flat, s=10, marker='o', c='k', label='_nolegend_', zorder=2) #, edgecolor=(1.,1.,1.,1.))
pylab.xlabel('$r$')
pylab.title('Film height')
pylab.ylim(0,2)
# pylab.legend(loc='upper left')



pylab.figure(figsize=(4,3), facecolor='w', edgecolor='k')
pylab.plot(r_linear, surfactant_true, color='0.0', linewidth=2, label='Surfactant', zorder=1)
pylab.scatter(r, surfactant_flat, s=10, c='k', label='_nolegend_', zorder=2)
pylab.xlabel('$r$')
pylab.title('Surfactant concentration')
# pylab.xlim(0,1)
pylab.ylim(-.02,.1201)
# pylab.legend(loc='upper right')



# pylab.savefig('/Users/onath/Documents/Projects/Surfactant/surfactant_paper/paper/figures/AxisymmetricScatter.eps')
