subroutine laplacian(q_laplacian, q, dx, dy, mx, my, mbc)

!----------------------------------------------------------
! Calculate Laplacian of q at cell centers, assuming that 
! the normal derivative of q vanishes at the boundaries.
!
! Computations are performed one vertical slice at a time,
! so we proceed through the array q in order of contiguous 
! memory.
!----------------------------------------------------------


    !$ use omp_lib


    !===> Variables ===>

    implicit none

    !---- Inputs ----
    integer, intent(in) :: mx, my, mbc
    double precision, intent(in) :: q(1-mbc:mx+mbc,1-mbc:my+mbc), dx, dy

    !---- Outputs ----
    double precision, intent(out) :: q_laplacian(1-mbc:mx+mbc,1-mbc:my+mbc)

    !---- Locals ----
    integer :: ix, iy

    !---- OpenMP ----
    !$ integer :: nthreads
    !$ common /omp_block/ nthreads

    !<=== Variables <===


    !$ call omp_set_num_threads(nthreads)
    


    !$omp parallel do private(ix)
    do iy = 1,my
        do ix = 1,mx

            q_laplacian(ix,iy) = (q(ix-1,iy) - 2.d0*q(ix,iy) + q(ix+1,iy)) / dx**2
            q_laplacian(ix,iy) = q_laplacian(ix,iy) + (q(ix,iy-1) - 2.d0*q(ix,iy) + q(ix,iy+1)) / dy**2

        end do
    end do
    


end subroutine laplacian