subroutine parity_extension(q, x_parity, y_parity, mx, my, mbc)

!---------------------------------------------------------------
! Extends the interior data of q into its ghost cells, based on
! either even or odd extension in both directions.  The inputs
! x_parity and y_parity should be set to 0 for even extension
! or 1 for odd extension, and need not agree.
!---------------------------------------------------------------


    !$ use omp_lib
    implicit none
    
    
    !===> Variables ===>
    
    !---- Inputs ----
    integer, intent(in) :: x_parity, y_parity, mx, my, mbc
    
    !---- In/out ----
    double precision, intent(inout) :: q(1-mbc:mx+mbc, 1-mbc:my+mbc)

    !---- Locals ----
    double precision :: x_factor, y_factor, xy_factor
    integer :: iy, iy_target
    !$ integer :: nthreads
    !$ common /omp_block/ nthreads
    
    !<=== Variables <===


    !$ call omp_set_num_threads(nthreads)


    !==== Set multiplicative factors for each type of extension ====
    x_factor  = (-1.d0)**x_parity
    y_factor  = (-1.d0)**y_parity
    xy_factor = x_factor * y_factor



    !==== Bottom, including corners ====
    !$omp parallel do private(iy_target)
    do iy = 1-mbc,0

        !---- Set y-index to copy from ----
        iy_target = 1-iy

        q(1-mbc:0,    iy) = xy_factor * q(mbc:1:-1,      iy_target)
        q(1:mx,       iy) =  y_factor * q(1:mx,          iy_target)
        q(mx+1:mx+mbc,iy) = xy_factor * q(mx:mx-mbc+1:-1,iy_target)

    end do



    !==== Left and right ====
    !$omp parallel do
    do iy = 1,my

        q(1-mbc:0,iy)     = x_factor * q(mbc:1:-1,      iy)
        q(mx+1:mx+mbc,iy) = x_factor * q(mx:mx-mbc+1:-1,iy)

    end do



    !==== Top, including corners ====
    !$omp parallel do private(iy_target)
    do iy = my+1,my+mbc

        !---- Set y-index to copy from ----
        iy_target = my - (iy-my) + 1

        q(1-mbc:0,    iy) = xy_factor * q(mbc:1:-1,      iy_target)
        q(1:mx,       iy) =  y_factor * q(1:mx,          iy_target)
        q(mx+1:mx+mbc,iy) = xy_factor * q(mx:mx-mbc+1:-1,iy_target)

    end do
    
end subroutine parity_extension