
""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

#--------------------------
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from pyclaw.plotters import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data

    #Defines a function to set axis labels (used by aftergrid)
    def get_set_axis_labels(xl,yl):
        def func(data):
            from pylab import xlabel,ylabel
            xlabel(xl)
            ylabel(yl)
        return func

    # pcolor of film height
    plotfigure = plotdata.new_plotfigure(name='Film Height', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-10., 10.]
    plotaxes.ylimits = [-4., 4.]
    plotaxes.title = 'Film height'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 0
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.pcolor_cmin = 0.5
    plotitem.pcolor_cmax = 1.5
    plotitem.add_colorbar = True

    plotitem.aftergrid = get_set_axis_labels('x','y')
    # pcolor of surfactant concentration
    plotfigure = plotdata.new_plotfigure(name='Surfactant Concentration', figno=2)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-4., 4.]
    plotaxes.ylimits = [-4., 4.]
    plotaxes.title = 'Surfactant concentration'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 1
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 1.0
    plotitem.add_colorbar = True
    
    plotitem.aftergrid = get_set_axis_labels('x','y')

    # # Figure for contour plot
    # plotfigure = plotdata.new_plotfigure(name='contour', figno=1)
    # 
    # # Set up for axes in this figure:
    # plotaxes = plotfigure.new_plotaxes()
    # plotaxes.xlimits = [-1., 2.]
    # plotaxes.ylimits = [-1., 2.]
    # plotaxes.title = 'Solution'
    # plotaxes.scaled = True
    # 
    # # Set up for item on these axes:
    # plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    # plotitem.plot_var = 0
    # plotitem.contour_nlevels = 20
    # plotitem.contour_min = 0.01
    # plotitem.contour_max = 0.99
    # plotitem.contour_colors = 'k'


    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

    
