
from setrun import setrun
from setplot import setplot
from pyclaw.runclaw import runclaw
from pyclaw.plotters.plotclaw import plotclaw

from numpy import *
import pylab

# initialize rundata using setrun but then change some things for each run:
rundata = setrun()
rundata.clawdata.dt_variable = 0
rundata.clawdata.nout = 1




#---- Baseline discretization parameters ----
# mx_base = 1000


#---- dt refinement parameters ----
# t_final_target = 0.5
# n_runs         = 41
# dt_base        = 0.05
# error_values   = zeros(n_runs)



#===> Solve for each dt ===>

def refinementTest( dt_values, (xlower,xupper), mx ):
    
    n_runs = len(dt_values)
    rundata.clawdata.xlower = xlower
    rundata.clawdata.xupper = xupper
    rundata.clawdata.mx     = mx
    
    error_values = zeros(n_runs)
    
    t_final_target = 1.0
    

    for i_run in range(n_runs):

        #---- Discretization parameters for this run ----
        dt      = dt_values[i_run]
        t_final = round(t_final_target/dt)*dt
    
        #---- Write parameters to clawdata ----
        rundata.clawdata.dt_initial = dt
        rundata.clawdata.tfinal     = t_final
        rundata.write()

        #---- Execute the run ----
        runclaw(xclawcmd = "xclaw", outdir="_output")
    
        #---- Read the final numerical error ----
        f = open("_output/error_value.txt", "r")
        error_values[i_run] = float(f.readline())
        f.close()
    
    return error_values
    
#<=== Solve for each dt <===


dt_values = .1 * logspace(0,-3,21)

# error_values_1 = refinementTest( dt_values, (-1., 1.), 400)

error_values = refinementTest( dt_values, (-3., 3.), 1200)





#===> Produce error plots ===>

plot_parameters = {'backend': 'eps',
          'lines.linewidth': 2,
          'lines.markersize': 4,
          'text.fontsize': 10,
          'font.size'    : 10,
          'legend.fontsize': 10,
          'xtick.labelsize': 10,
          'ytick.labelsize': 10,
          'axes.labelsize':  14,
          'figure.subplot.left':   0.15,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.15,
          'figure.subplot.top':    0.9}
          
pylab.rcParams.update(plot_parameters)

pylab.figure(figsize=(4,3), facecolor='w', edgecolor='k')


# pylab.clf()
pylab.loglog(dt_values, error_values, 'ko', label="Error ($\infty$-norm)")

pylab.loglog(dt_values, dt_values,    'k-', label="$\Delta t$")

pylab.xlim(1e-4/2, 2e-1)

pylab.xlabel('$\Delta t$')
# pylab.title('Numerical errors for the KdV equation')
pylab.legend(loc='upper left')


pylab.savefig('/Users/onath/Documents/Projects/Surfactant/surfactant_paper/paper/figures/KdVErrors.eps')


#<=== Produce error plots <===