subroutine kdv_implicit_step( q, q_newton, dq, new_bcs, tolerance, &
                              dx, dt, mx, mbc, success )


    !===> Variables ===>

    implicit none

    !---- Inputs ----
    integer, intent(in) :: mx, mbc
    double precision, intent(in) :: new_bcs(4), tolerance, dx, dt

    !---- In/out ----
    double precision, intent(inout) :: q(1-mbc:mx+mbc)

    !---- Output ----
    !-------------------------------------------------------------
    ! The output values of the arrays here are not actually used.
    ! The intent just indicates that their input values do not
    ! matter.
    !-------------------------------------------------------------
    double precision, intent(out), dimension(1-mbc:mx+mbc) :: q_newton, dq
    logical, intent(out) :: success

    !---- Globals ----
    integer :: newton_max_iter
    double precision :: newton_reduction_factor
    common /newton_block/ newton_max_iter, newton_reduction_factor


    !---- Locals ----
    double precision :: norm_dq, old_norm_dq
    integer :: iter, ix
    !$ integer :: nthreads
    !$ common /comomp/ nthreads
    
    !<==== Variables <===



    !---- Initialize ----
    print *, "Entering kdv_parabolic_step with dt = ", dt
    

    do ix = 1,mx
        q_newton(ix) = q(ix)
    end do

    q_newton(-1)   = new_bcs(1)
    q_newton(0)    = new_bcs(2)
    q_newton(mx+1) = new_bcs(3)
    q_newton(mx+2) = new_bcs(4)
    
    success     = .false.
    norm_dq     = tolerance + 1.d0
    old_norm_dq = norm_dq
    iter        = 0
    

    !===> Newton iterations ===>

    do while (norm_dq > tolerance)
    
        !---- Iteration count ----
    
        iter = iter + 1
        
        !---- Calculate RHS for Newton's method; store in dq ----

        call kdv_newton_rhs( dq, q_newton, q, dx, dt, mx, mbc )


        
        !---- Solve, overwriting dq ----

        call kdv_integrate_slice( dq(1:mx), q_newton(0:mx+1), dx, dt, mx )
        
        
        
        
        !---- Update q_newton and norm_dq ----
        old_norm_dq = norm_dq
        norm_dq     = 0.d0

        do ix = 1,mx

            q_newton(ix) = q_newton(ix) + dq(ix)

            norm_dq      = max( norm_dq, abs(dq(ix)) )

        end do
        



        !---- Give up if norm_dq is too big ----
        if (norm_dq > 1e6) exit


        !---- If we've converged, then finish ----
        if (norm_dq < tolerance) then

            do ix = 1,mx
                q(ix) = q_newton(ix)
            end do

            success = .true.
            exit
        end if


        !---- If we've hit the max # of iterations, give up unless we're making sufficient progress ----

        if ( (iter >= newton_max_iter) .and. (norm_dq > newton_reduction_factor*old_norm_dq) ) exit


    end do
    !<=== Newton iterations <===




end subroutine kdv_implicit_step