subroutine kdv_integrate_slice( rhs, q, dx, dt, mx )



    implicit none


    !===> Variables ===>

    !---- Inputs ----
    integer, intent(in) :: mx
    double precision, intent(in) :: q(0:mx+1)
    double precision, intent(in) :: dx, dt

    !---- In/out ----
    double precision, intent(inout) :: rhs(mx)

    !---- Locals ----
    integer :: ipiv(mx), info, i, j, n_lower, n_upper
    parameter (n_lower=2, n_upper=2)
    double precision :: backward_euler(2*n_lower+1+n_upper, mx), flux(n_lower+n_upper,mx)

    !<=== Variables <===



    !---- Calculate flux operator ----

    call kdv_sparse_flux( flux, q, dx, dt, mx )



    !---- Define backward_euler ----
    !--------------------------------------------------------------------------------
    ! The body of the loop implements
    !
    !       backward_euler(2,j) =        dt/dx *  flux(1,j)
    !       backward_euler(3,j) = 1.d0 + dt/dx * (flux(2,j) - flux(1,j))
    !       backward_euler(4,j) =        dt/dx * (                 - flux(2,j))
    !
    ! in a form that can be used with problems of other orders as well.
    !--------------------------------------------------------------------------------

    do j = 1, mx

        backward_euler(n_lower+1,j) = dt/dx * flux(1,j)


        do i = 2, n_lower+n_upper
            backward_euler(n_lower+i,j) = dt/dx * (flux(i,j) - flux(i-1,j))
        end do

        backward_euler(2*n_lower+1,j) = backward_euler(2*n_lower+1,j) + 1.d0


        backward_euler(2*n_lower+n_upper+1,j) = dt/dx * ( -flux(n_lower+n_upper,j) )

    end do




    !---- Solve; rhs is overwritten with the result ----
    
    call dgbsv(mx, n_lower, n_upper, 1, backward_euler, 2*n_lower+n_upper+1, ipiv, rhs, mx, info)


end subroutine kdv_integrate_slice