subroutine kdv_newton_rhs(rhs, q, q_old, dx, dt, mx, mbc)

!===> Description ===>
!
! Evaluates the RHS of Newton's method for the parabolic part of
! the porous medium equation.
!
!<=== Description <===


    !$ use omp_lib
    implicit none


    !===> Variables ===>

    !---- Inputs ----
    integer, intent(in) :: mx, mbc
    double precision, intent(in), dimension(1-mbc:mx+mbc) :: q, q_old
    double precision, intent(in) :: dx, dt

    !---- Outputs ----
    double precision, intent(out), dimension(1-mbc:mx+mbc) :: rhs

    !---- Locals ----
    integer :: ix
    double precision :: flux(mx+1), q_face, q_xx

    !<=== Variables <===


    

    !---- First part of the RHS is easy ----
    
    rhs(1:mx) = q_old(1:mx) - q(1:mx)



    !---- Calculate flux at each interface ----
    !--------------------------------------------------------
    ! Ghost cell values are used at the boundary interfaces.
    !--------------------------------------------------------

    do ix = 1,mx+1
        
        q_face   = (q(ix-1) + q(ix)) / 2.d0
        q_xx     = .5d0 * (q(ix-2) - q(ix-1) - q(ix) + q(ix+1)) / dx**2
        
        flux(ix) = -.75d0*q_face**2 - .25d0*q_xx
        
    end do

    
    
    
    !---- Update RHS with fluxes ----
    
    do ix = 1, mx
        
        rhs(ix) = rhs(ix) - dt/dx * (flux(ix+1) - flux(ix))

    end do



end subroutine kdv_newton_rhs
