subroutine kdv_sparse_flux( sparse_flux, q, dx, dt, mx )


    !===> Variables ===>

    implicit none

    !---- Inputs ----
    integer, intent(in) :: mx
    double precision, intent(in) :: q(0:mx+1)
    double precision, intent(in) :: dx, dt

    !---- Outputs ----
    double precision, intent(out) :: sparse_flux(4,mx)

    !---- Locals ----
    double precision :: q_face, q_x, f, fd1, one_eighth
    integer :: j
    parameter( one_eighth = 1.d0/8.d0 )

    !<==== Variables <====



    !#NEW
    sparse_flux(1,1) = 0.d0


    !---- Interface 1 ----
    
    j = 1
    
    q_face        = (q(j-1) + q(j)) / 2.d0
    
!     sparse_flux(4,j-2) = -one_eighth / dx**2
!     sparse_flux(3,j-1) =  one_eighth / dx**2 - .75*q_face
    sparse_flux(2,j)   =  one_eighth / dx**2 - .75*q_face
    sparse_flux(1,j+1) = -one_eighth / dx**2



    !---- Interface 2 ----
    !-------------------------------------------------------------------------
    ! The component in position (3,1), which is the influence of cell 1 on
    ! interface 2, is affected by the even extension of h across the boundary.
    ! This effectively sets h(0)=h(1), and means that we see 2.d0*kappa/dx**3
    ! in place of 3.d0*kappa/dx**3.
    !-------------------------------------------------------------------------
 
    j = 2
    
    q_face        = (q(j-1) + q(j)) / 2.d0
    
!     sparse_flux(4,j-2) = -one_eighth / dx**2
    sparse_flux(3,j-1) =  one_eighth / dx**2 - .75*q_face
    sparse_flux(2,j)   =  one_eighth / dx**2 - .75*q_face
    sparse_flux(1,j+1) = -one_eighth / dx**2


    !<=== Lower boundary <===



    !===> Interior interfaces ===>

    do j = 3,mx-1

        q_face        = (q(j-1) + q(j)) / 2.d0
        
        sparse_flux(4,j-2) = -one_eighth / dx**2
        sparse_flux(3,j-1) =  one_eighth / dx**2 - .75*q_face
        sparse_flux(2,j)   =  one_eighth / dx**2 - .75*q_face
        sparse_flux(1,j+1) = -one_eighth / dx**2

    end do

    !<=== Interior interfaces <===    



    !===> Upper boundary ===>

    !---- Interface mx ----
    !----------------------------------------------------------------
    ! Like element (3,1), the component in (2,mx) is affected by the 
    ! even extension across teh boundary, which sets h(mx+1)=h(mx).
    !----------------------------------------------------------------
    
    j = mx
    
    q_face        = (q(j-1) + q(j)) / 2.d0
    
    sparse_flux(4,j-2) = -one_eighth / dx**2
    sparse_flux(3,j-1) =  one_eighth / dx**2 - .75*q_face
    sparse_flux(2,j)   =  one_eighth / dx**2 - .75*q_face
!     sparse_flux(1,j+1) = -one_eighth / dx**2



    !---- Interface mx+1 ----

    j = mx+1

    q_face        = (q(j-1) + q(j)) / 2.d0
    
    sparse_flux(4,j-2) = -one_eighth / dx**2
    sparse_flux(3,j-1) =  one_eighth / dx**2 - .75*q_face
!     sparse_flux(2,j)   =  one_eighth / dx**2 - .75*q_face
!     sparse_flux(1,j+1) = -one_eighth / dx**2



    !#NEW
    sparse_flux(4,mx) = 0.d0

    !<=== Upper boundary <===
    


end subroutine kdv_sparse_flux