subroutine porous_sparse_flux( sparse_flux, q, dx, dt, mx )


    !===> Variables ===>

    implicit none

    !---- Inputs ----
    integer, intent(in) :: mx
    double precision, intent(in) :: q(0:mx+1)
    double precision, intent(in) :: dx, dt

    !---- Outputs ----
    double precision, intent(out) :: sparse_flux(2,mx)

    !---- Locals ----
    double precision :: q_face, q_x, f, fd1
    integer :: j

    !<==== Variables <====



    !---- Lower boundary ----
    !-----------------------------------------------------------
    ! This is calculating a dq with a zero left boundary value.
    !-----------------------------------------------------------

    q_face = (q(1) + q(0)) / 2.d0

    q_x    = (q(1) - q(0)) / dx
    f      = -q_face
    fd1    = -1.d0

    sparse_flux(1,1)   = fd1*q_x/2.d0 + f/dx
    
    
    !---- Interior interfaces ----
    
    do j = 2, mx

        q_face = (q(j-1) + q(j)) / 2.d0

        q_x    = (q(j) - q(j-1)) / dx
        f      = -q_face
        fd1    = -1.d0
        
        sparse_flux(2,j-1) = fd1*q_x/2.d0 - f/dx
        sparse_flux(1,j)   = fd1*q_x/2.d0 + f/dx

    end do



    !---- Upper boundary ----

    q_face = (q(mx) + q(mx+1)) / 2.d0

    q_x    = (q(mx+1) - q(mx)) / dx
    f      = -q_face
    fd1    = -1.d0

    sparse_flux(2,mx) = fd1*q_x/2.d0 - f/dx
    
    


end subroutine porous_sparse_flux