subroutine src1(maxmx, meqn, mbc, mx, x_low, dx, q, maux, aux, t, dt)

    implicit none

    !==== Variables ====
    
    !---- Inputs ----
    integer, intent(in) :: maxmx, meqn, mbc, mx, maux
    double precision, intent(in) :: x_low, dx, t, dt
    double precision, intent(in) :: aux(1-mbc:maxmx+mbc, maux)
    
    !---- Input/output ----
    double precision, intent(inout) :: q(1-mbc:maxmx+mbc, meqn)

    !---- Functions ----
    double precision :: q_true

    !---- Locals ----
    integer :: n_splits, max_splits, i
    double precision :: t_new, t_local, dt_local, tolerance, new_bcs(2), x_coordinate
    logical :: success
    
    
    !==== Statement function ====
    x_coordinate(i) = x_low + (i-.5d0)*dx
    
    
    !---- Set tolerance for Newton's method ----
    !------------------------------------------------------------
    ! I'd like to set this to dt**2, but that causes problems if
    ! the value is too small.
    !------------------------------------------------------------
    tolerance = dmax1(dt**2, 1d-12)
    
    
    
    max_splits = 4
    

    !---- Clawpack currently gives src1 the time at the *end* of the step ----
    t_new = t

    t_local  = t_new-dt
    dt_local = dt
    n_splits = 0
    success  = .false.

!     print *, t_new - t_local

    do while ( t_new - t_local > 1d-8)
    
        new_bcs(1) = q_true( x_low-dx/2,           t_local+dt_local )
        new_bcs(2) = q_true( x_low + mx*dx + dx/2, t_local+dt_local )
        call porous_parabolic_step( q(:,1), aux(:,1), aux(:,2), new_bcs, tolerance, &
                                    dx, dt_local, mx, mbc, success)


        if (success) then
            t_local  = t_local + dt_local
        else
            dt_local = dt_local / 2.d0
            n_splits = n_splits + 1
            print *, 'Splitting time step'

            if (n_splits > max_splits) then
                print '(A)', 'Parabolic step failed with maximum number of dt splits; aborting.'
                stop
            end if
        end if
    
    end do
    
    
    
    
    !---- Store true solution in a second variable ----
    do i = 1, mx
        q(i,2) = q_true( x_coordinate(i), t_new )
    end do
    
    


end
