subroutine b4step2(maxmx,maxmy,mbc,mx,my,meqn,q,xlower,ylower,dx,dy,t,dt,maux,aux)

!-----------------------------------------------------------------
! Called from claw2 before each call to step2.  Used to set the 
! aux arrays with velocity field data.
!
!    aux(:,:,1) = film velocity field, horizontal component
!    aux(:,:,2) = film velocity field, vertical component
!    aux(:,:,3) = surfactant velocity field, horizontal component
!    aux(:,:,4) = surfactant velocity field, vertical component
!-----------------------------------------------------------------


    !$ use omp_lib
    implicit none


    !===> Variables ===>

    !---- Globals ----
    double precision :: beta, kappa, mu, delta
    common /param_block/    beta, kappa, mu, delta

    !---- Inputs ----
    integer, intent(in) :: maxmx, maxmy, mbc, mx, my, meqn, maux
    double precision, intent(in) :: xlower, ylower, dx, dy, t, dt
    double precision, intent(in) :: q(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc, meqn)

    !---- Outputs ----
    double precision, intent(out) :: aux(1-mbc:maxmx+mbc,1-mbc:maxmy+mbc, maux)

    !---- Locals ----
    double precision, dimension(1-mbc:mx+mbc) :: sigma1
    double precision, dimension(1-mbc:my+mbc) :: sigma2
    integer :: ix, iy, iaux
    double precision :: h_laplacian(1-mbc:mx+mbc,1-mbc:my+mbc)
    double precision :: h_lap_2(1-mbc:mx+mbc,1-mbc:my+mbc)
        
    double precision :: hface, h_x, h_laplacian_x, h_y, h_laplacian_y, gx, gy
    double precision :: tension_d1(mx)
    !$ integer :: nthreads
    !$ common /omp_block/ nthreads

    !<=== Variables <===

    
    
    !$ call omp_set_num_threads(nthreads)
    
    
    
    !===> Film velocity field, x-component ===>

    !---- Fill interior cells ----
    !$omp parallel do private(tension_d1, ix, gx)
    do iy=1,my
        
        !---- Set surface tension on horizontal slice ----
        tension_d1(1:mx) = q(1:mx,iy,2)
        call surface_tension_d1(tension_d1,mu,mx)
      
        do ix=1,mx
            gx = (q(ix+1,iy,2) - q(ix-1,iy,2)) / (2.d0*dx)
            aux(ix,iy,1) = tension_d1(ix) * gx
        end do
                
    end do  

    !---- Fill ghost cells ----
    call extend_to_ghosts( aux(:,:,1), 'odd', 'periodic', mx, my, mbc )

    !<=== Film velocity field, x-component <===
    
    
    
    !===> Film velocity field, y-component ===>
    
    !---- Fill interior cells ----
    !$omp parallel do private(tension_d1, ix, gy)
    do iy=1,my
        tension_d1(1:mx) = q(1:mx,iy,2)
        call surface_tension_d1(tension_d1,mu,mx)
        
        do ix=1,mx
            gy = (q(ix,iy+1,2) - q(ix,iy-1,2)) / (2.d0*dy)
            aux(ix,iy,2) = tension_d1(ix) * gy
        end do
    end do
        
    !---- Fill ghost cells ----
    call extend_to_ghosts( aux(:,:,2), 'even', 'periodic', mx, my, mbc )
    
    !<=== Film velocity field, y-component <===



    !==== Evaluate Laplacian of h; used for surfactant velocity field ====
    call extend_to_ghosts( q(:,:,1), 'even', 'periodic', mx, my, mbc )
    call laplacian(h_laplacian, q(:,:,1), dx, dy, mx, my, mbc)
    call extend_to_ghosts( h_laplacian, 'even', 'periodic', mx, my, mbc )

    
    !===> Surfactant velocity field, y-component ===>

    !---- Fill interior cells ----
    !$omp parallel do private(ix, h_x, h_laplacian_x)
    do iy=1,my
        do ix=1,mx

            h_x           = (q(ix+1,iy,1)  - q(ix-1,iy,1))  / (2.d0*dx)
            h_laplacian_x = (h_laplacian(ix+1,iy) - h_laplacian(ix-1,iy)) / (2.d0*dx)

            aux(ix,iy,3) = q(ix,iy,1)**2/2.d0 * (-beta*h_x + kappa*h_laplacian_x)

        end do                    
    end do
    
    !---- Fill ghost cells ----
    call extend_to_ghosts( aux(:,:,3), 'odd', 'periodic', mx, my, mbc )

    !<=== Surfactant velocity field, x-component <===
  
  
  
    !===> Surfactant velocity field, y-component ===>

    !---- Fill interior cells ----
    !$omp parallel do private(ix, h_y, h_laplacian_y)
    do iy=1,my
        do ix=1,mx

            h_y    = (q(ix,iy+1,1) -  q(ix,iy-1,1))  / (2.d0*dy)
            h_laplacian_y = (h_laplacian(ix,iy+1) - h_laplacian(ix,iy-1)) / (2.d0*dy)

            aux(ix,iy,4) = q(ix,iy,1)**2/2.d0 * (-beta*h_y + kappa*h_laplacian_y)

        end do
    end do
    
    
    !---- Fill ghost cells ----
    call extend_to_ghosts( aux(:,:,4), 'even', 'periodic', mx, my, mbc )
    
    !<=== Surfactant velocity field, y-component <===







    

    
    return
end subroutine b4step2
