subroutine rpn2(normal_direction,maxm,meqn,mwaves,mbc,mx,ql,qr,auxl,auxr,wave,s,amdq,apdq)

!--------------------------------------------------------------------------
! This solves Riemann problems for the thin film/surfactant model.
! The film (h=q(:,:,1)) and surfactant (Gamma=q(:,:,2)) act independently.
! On input, ql contains the state vector at the left edge of each cell,
! and qr contains the state vector at the right edge of each cell.
!
! The film advects with a Burgers-type equation,
!     h_t + (1/2 * h**2 * sigma_x)_x + (1/2 * h**2 * sigma_y)_y = 0.
!
! In b4step2, sigma_x and sigma_y have been stored on each cell, with
!
!     aux(i,j,1) = sigma_x(i,j), and
!     aux(i,j,2) = sigma_y(i,j).
!
!
! The surfactant evolves by simple advection,
!     Gamma_t + (-1/2 * beta * h^2 * h_x * Gamma)_x 
!             + (-1/2 * beta * h^2 * h_y * Gamma)_y = 0.
!
! The velocities have been stored in the aux arrays, with
!
!     aux(i,j,3) = -1/2 * beta * h^2 * h_x(i,j)
!     aux(i,j,4) = -1/2 * beta * h^2 * h_y(i,j).
!--------------------------------------------------------------------------


    implicit none


    !===> Variables ===>

    !---- Inputs ----
    integer, intent(in) :: normal_direction, maxm, meqn, mwaves, mbc, mx
    double precision, intent(in) ::   ql(1-mbc:maxm+mbc, meqn)
    double precision, intent(in) ::   qr(1-mbc:maxm+mbc, meqn)
    double precision, intent(in) :: auxl(1-mbc:maxm+mbc, *)
    double precision, intent(in) :: auxr(1-mbc:maxm+mbc, *)

    !---- Outputs ----
    double precision, intent(out) :: wave(1-mbc:maxm+mbc, meqn, mwaves)
    double precision, intent(out) ::    s(1-mbc:maxm+mbc, mwaves)
    double precision, intent(out) :: apdq(1-mbc:maxm+mbc, meqn)
    double precision, intent(out) :: amdq(1-mbc:maxm+mbc, meqn)

    !---- Locals ----
    integer :: i
    double precision :: u_lower, u_upper

    !<=== Variables <===




    s = 0.d0
    wave = 0.d0
    amdq = 0.d0
    apdq = 0.d0
    




    return
end subroutine rpn2