subroutine film_newton_rhs(rhs, h, h_laplacian, h_old, beta, kappa, dx, dy, dt, mx, my, mbc)

!===> Description ===>
!
! Evaluating that sort of nasty right-hand side
! for Newton's method.
!
!<=== Description <===


    !$ use omp_lib
    implicit none


    !===> Variables ===>
    
    !---- Inputs ----
    integer, intent(in) :: mx, my, mbc
    double precision, intent(in), dimension(1-mbc:mx+mbc,1-mbc:my+mbc) :: h, h_laplacian, h_old
    double precision, intent(in) :: beta, kappa, dx, dy, dt
    
    !---- Outputs ----
    double precision, intent(out), dimension(1-mbc:mx+mbc,1-mbc:my+mbc) :: rhs
    !---- Locals ----
    integer :: ix, iy
    double precision :: flux(mx+1), h_face
    !$ integer :: nthreads
    !$ common /omp_block/ nthreads

    !<=== Variables <===


    !---- Initialize interior values of rhs ----

    rhs(1:mx,1:my) = h_old(1:mx,1:my) - h(1:mx,1:my)




    !==== Flux contributions of fixed-x interfaces ====

    !$omp parallel do private(ix, h_face, flux)
    do iy = 1,my
        
        !---- Calculate fluxes ----
        
        do ix = 1,mx+1
            h_face   = (h(ix-1,iy) + h(ix,iy)) / 2.d0
            flux(ix) = h_face**3/3.d0 * (kappa * (h_laplacian(ix,iy) - h_laplacian(ix-1,iy)) / dx  &
                                            - beta * (h(ix,iy) - h(ix-1,iy)) / dx                  )
        end do


        !---- Update RHS ----
        
        do ix = 1, mx
            rhs(ix,iy) = rhs(ix,iy) - dt/dx * (flux(ix+1) - flux(ix))
        end do
        
    end do


    
    
    !==== Flux contributions of fixed-y interfaces ====
    
    !$omp parallel do private(ix, h_face, flux)
    do iy = 1, my+1
        
        !---- Calculate fluxes ----
        
        do ix = 1, mx  
            h_face   = (h(ix,iy-1) + h(ix,iy)) / 2.d0
            flux(ix) = h_face**3/3.d0 * (kappa * (h_laplacian(ix,iy) - h_laplacian(ix,iy-1)) / dy  &
                                                      - beta * (h(ix,iy) - h(ix,iy-1)) / dy        )
        end do
        
        
        !---- Update RHS ----

        if (iy > 1) then
            do ix = 1,mx          
                rhs(ix,iy-1) = rhs(ix,iy-1) - dt/dy * flux(ix)
            end do
        end if
        
        if (iy <= my) then
            do ix = 1, mx
                rhs(ix,iy) = rhs(ix,iy) + dt/dy * flux(ix)
            end do
        end if        
        
    end do



end subroutine film_newton_rhs