subroutine src2(maxmx,maxmy,meqn,mbc,mx,my,xlower,ylower,dx,dy,q,maux,aux,t,dt)

!-----------------------------------------------------------------
! Controls integration of the parabolic terms.  The main content
! of this routine is the adaptive time stepping, which halves the
! time step for a parabolic integration routine in the event that
! Newton's method fails to converge.
!-----------------------------------------------------------------

    implicit none


    !===> Variables ===>

    !---- Globals ----
    double precision :: beta, kappa, mu, delta
    common /param_block/    beta, kappa, mu, delta

    !---- Inputs ----
    integer, intent(in) :: maxmx, maxmy, meqn, mbc, mx, my, maux
    double precision, intent(in) :: xlower, ylower, dx, dy, t, dt
    double precision, intent(in) :: aux(1-mbc:maxmx+mbc, 1-mbc:maxmy+mbc, maux)

    !---- In/out ----
    double precision, intent(inout) :: q(1-mbc:maxmx+mbc, 1-mbc:maxmy+mbc, meqn)

    !---- Locals ----    
    integer :: ix, iy, max_splits, n_splits, step
    double precision :: tolerance, t_local, dt_local
    logical :: success
!     double precision :: periodic_tol = 1.d-8

    !<=== Variables <===



    !---- Set tolerance for Newton's method ----
    !------------------------------------------------------------
    ! I'd like to set this to dt**2, but that causes problems if
    ! the value is too small.
    !------------------------------------------------------------
    tolerance = dmax1(dt**2, 1d-12)
!     tolerance = 1.d-4

    max_splits = 8


    t_local  = t
    dt_local = dt
    n_splits = 0
    success  = .false.
    
    
    do while (abs(t_local - (t+dt)) > 1d-8)
        call film_parabolic_step( q(:,:,1), aux(:,:,1), aux(:,:,2), aux(:,:,3), tolerance, &
                                  beta, kappa, dx, dy, dt_local, mx, my, mbc, success)
        
        if (success) then
            t_local  = t_local + dt_local
        else
            dt_local = dt_local / 2.d0
            n_splits = n_splits + 1
            print *, 'Splitting time step (film)'
        
            if (n_splits > max_splits) then
                print '(A)', 'Film parabolic step failed with maximum number of dt splits; aborting.'
                stop
            end if
        end if
    end do







    !---- Surfactant step ----
    t_local  = t
    dt_local = dt
    n_splits = 0
    success  = .false.

    do while (abs(t_local - (t+dt)) > 1d-8)
        call surfactant_parabolic_step( q(:,:,2), q(:,:,1), aux(:,:,1), aux(:,:,2), tolerance, &
                                        delta, mu, dx, dy, dt_local, mx, my, mbc, success       )
                                           
        if (success) then
            t_local  = t_local + dt_local
        else
            dt_local = dt_local / 2.d0
            n_splits = n_splits + 1
            print *, 'Splitting time step (surfactant)'
    
            if (n_splits > max_splits) then
                print '(A)', 'Surfactant parabolic step failed with maximum number of dt splits; aborting.'
                stop
            end if
        end if
    end do



    return
end subroutine src2
