subroutine surfactant_integrate_slice_periodic(rhs, g, h, delta, mu, dx, dt, mx)

!---------------------------------------------------------
! Solves a 1d problem for the ADI split on the surfactant
! equation.
!---------------------------------------------------------


    implicit none


    !===> Variables ===>

    !---- Inputs ----
    integer, intent(in) :: mx
    double precision, intent(in) :: g(mx), h(mx)
    double precision, intent(in) :: delta, mu, dx, dt

    !---- In/out ----
    double precision, intent(inout) :: rhs(mx)

    !---- Locals ----
    integer :: ipiv(mx), info, i, j, ndiagl, ndiagu
    parameter (ndiagl=1, ndiagu=1)
    double precision :: backward_euler(2*ndiagl+1+ndiagu, mx), sparse_flux(ndiagl+ndiagu,mx)

    !<=== Variables <===



    !---- Set sparse_flux ----
    call surfactant_sparse_flux_periodic( sparse_flux, g, h, delta, mu, dx, dt, mx )



    !---- Lower boundary ----
    backward_euler(2,1) =        dt/dx *  sparse_flux(1,1)                   
    backward_euler(3,1) = 1.d0 + dt/dx * (sparse_flux(2,1) - sparse_flux(1,1))
    backward_euler(4,1) =        dt/dx * (                 - sparse_flux(2,1))



    !---- Interior cells ----
    do j=2,mx-1

      backward_euler(2,j) =        dt/dx *  sparse_flux(1,j)
      backward_euler(3,j) = 1.d0 + dt/dx * (sparse_flux(2,j) - sparse_flux(1,j))
      backward_euler(4,j) =        dt/dx * (                 - sparse_flux(2,j))

    end do



    !---- Upper boundary ----
    backward_euler(2,mx) =        dt/dx *  sparse_flux(1,mx)
    backward_euler(3,mx) = 1.d0 + dt/dx * (sparse_flux(2,mx) - sparse_flux(1,mx))
    backward_euler(4,mx) =      + dt/dx * (                  - sparse_flux(2,mx))



    !---- Solve; rhs is overwritten with the result ----
!     call dgbsv(mx, ndiagl, ndiagu, 1, backward_euler, 2*ndiagl+ndiagu+1, ipiv, rhs, mx, info)
    call periodic_banded_solve( backward_euler, rhs, mx-1, 1, 1, 1 )


end subroutine surfactant_integrate_slice_periodic