subroutine surfactant_sparse_flux(sparse_flux, g, h, delta, mu, dx, dt, mx)

!===> Description ===>
!
! Calculates derivatives of the flux function for a 1d
! Newton iteration.
!
!<=== Description <===

    ! use_omp_lib

    !===> Variables ===>

    implicit none

    !---- Inputs ----
    integer, intent(in) :: mx
    double precision, intent(in) :: g(mx), h(mx)
    double precision, intent(in) :: delta, mu, dx, dt

    !---- Outputs ----
    double precision, intent(out) :: sparse_flux(2,mx)

    !---- Locals ----
    double precision :: h_face, g_face, g_x, f, fd1
    double precision :: tension_d1(mx), tension_d2(mx)
    integer :: j

    !<==== Variables <====



    !---- Lower boundary ----
    sparse_flux(1,1) = 0.d0
    
    
    !---- Interior interfaces ----
    do j=2,mx
        tension_d1(j) = (g(j-1) + g(j))/2.d0
        tension_d2(j) = tension_d1(j)
    end do
    call surface_tension_d1(tension_d1, mu, mx)
    call surface_tension_d2(tension_d2, mu, mx)
    
    do j=2,mx

        h_face = (h(j-1) + h(j)) / 2.d0
        g_face = (g(j-1) + g(j)) / 2.d0
        g_x    = (g(j) - g(j-1)) / dx
        f      = h_face * g_face * tension_d1(j) - delta
        fd1    = h_face * (tension_d1(j) + g_face*tension_d2(j))
        
        sparse_flux(2,j-1) = fd1*g_x/2.d0 - f/dx
        sparse_flux(1,j)   = fd1*g_x/2.d0 + f/dx

    end do


    !---- Upper boundary ----
    sparse_flux(2,mx) = 0.d0

end subroutine surfactant_sparse_flux