subroutine surfactant_sparse_flux_periodic ( sparse_flux, g, h, delta, mu, dx, dt, mx )

!===> Description ===>
!
! Calculates derivatives of the flux function for a 1d
! Newton iteration.
!
!<=== Description <===

    ! use_omp_lib

    !===> Variables ===>

    implicit none

    !---- Inputs ----
    integer, intent(in) :: mx
    double precision, intent(in) :: g(mx), h(mx)
    double precision, intent(in) :: delta, mu, dx, dt

    !---- Outputs ----
    double precision, intent(out) :: sparse_flux(2,mx)

    !---- Locals ----
    double precision :: h_face, g_face, g_x, f, fd1
    double precision :: tension_d1(mx), tension_d2(mx)
    integer :: j

    !<==== Variables <====




    
    !==== Calculate surface tension derivatives ====
    
    !---- Initialize arrays to Gamma values at each interface ----
    tension_d1(1) = (g(mx) + g(1)) / 2.d0
    do j=2,mx
        tension_d1(j) = (g(j-1) + g(j))/2.d0
    end do
    tension_d2 = tension_d1

    !---- Compute ----
    call surface_tension_d1(tension_d1, mu, mx)
    call surface_tension_d2(tension_d2, mu, mx)

    


    !---- Interface 1, with periodic effects ----
    h_face = (h(mx) + h(1)) / 2.d0
    g_face = (g(mx) + g(1)) / 2.d0
    g_x    = (g(1) - g(mx)) / dx
    f      = h_face * g_face * tension_d1(1) - delta
    fd1    = h_face * (tension_d1(1) + g_face*tension_d2(1))
    
    sparse_flux(2,mx) = fd1*g_x/2.d0 - f/dx
    sparse_flux(1,1)  = fd1*g_x/2.d0 + f/dx




    !---- Interior interfaces ----
    do j=2,mx

        h_face = (h(j-1) + h(j)) / 2.d0
        g_face = (g(j-1) + g(j)) / 2.d0
        g_x    = (g(j) - g(j-1)) / dx
        f      = h_face * g_face * tension_d1(j) - delta
        fd1    = h_face * (tension_d1(j) + g_face*tension_d2(j))
        
        sparse_flux(2,j-1) = fd1*g_x/2.d0 - f/dx
        sparse_flux(1,j)   = fd1*g_x/2.d0 + f/dx

    end do



end subroutine surfactant_sparse_flux_periodic