subroutine qinit(maxmx, maxmy, meqn, mbc, mx, my, x_low, y_low, dx, dy, q)

!-------------------------------------------
! Sets initial conditions for:
!     film height = q(:,:,1);
!     surfactant concentration = q(:,:,2).
!-------------------------------------------

    implicit none

    !---> Variables --->
    ! Input
    integer, intent(in) :: maxmx, maxmy, meqn, mbc, mx, my
    double precision, intent(in) :: x_low, y_low, dx, dy

    ! Output
    double precision, intent(out) :: q(1-mbc:maxmx+mbc, 1-mbc:maxmy+mbc, meqn)

	! Global
	double precision :: beta, kappa, mu, delta
	common /param_block/	beta, kappa, mu, delta

    ! Local
    integer :: ix, iy
    double precision :: x, y, r
    !<--- Variables <---

    
    do iy = 1,my
        y = y_low + (iy-0.5d0)*dy
        do ix = 1,mx
            x = x_low + (ix-0.5d0)*dx

            ! Film height
            q(ix,iy,1) = 1.d0
            
            ! Surfactant concentration
            r = sqrt(x**2 + y**2)
            if (r <= 1.d0) then
                q(ix,iy,2) = 2*(1.d0 - r**10)
            else
                q(ix,iy,2) = 0.d0
            end if
        end do
    end do


    
end subroutine qinit
