subroutine extend_to_ghosts ( q, x_ext_type, y_ext_type, mx, my, mbc )

!---------------------------------------------------------------
! Extends the interior data of q into its ghost cells, based on
! either even or odd extension in both directions.  The inputs
! x_ext_type and y_ext_type should be set to 0 for even extension
! or 1 for odd extension, and need not agree.
!---------------------------------------------------------------


    !$ use omp_lib
    implicit none
    
    
    !===> Variables ===>
    
    !---- Inputs ----
    integer, intent(in) :: mx, my, mbc
    character*1 :: x_ext_type, y_ext_type
    
    !---- In/out ----
    double precision, intent(inout) :: q(1-mbc:mx+mbc, 1-mbc:my+mbc)

    !---- Locals ----
    integer :: ix, iy, iy_target
    logical :: lsame
    !$ integer :: nthreads
    !$ common /omp_block/ nthreads
    
    !<=== Variables <===


    !$ call omp_set_num_threads(nthreads)



    !===> Perform x-extension ===>
    
    !---- Even extension ----

    if ( lsame(x_ext_type, 'e') ) then
        
        !$omp parallel do
        do iy = 1, my
            
            q(1-mbc:0,iy)     = q(mbc:1:-1,      iy)
            q(mx+1:mx+mbc,iy) = q(mx:mx-mbc+1:-1,iy)
            
        end do

    !---- Odd extension ----

    else if ( lsame(x_ext_type, 'o') ) then
        
        !$omp parallel do
        do iy = 1, my
            
            q(1-mbc:0,iy)     = -q(mbc:1:-1,      iy)
            q(mx+1:mx+mbc,iy) = -q(mx:mx-mbc+1:-1,iy)
            
        end do
        
    !---- Periodic extension ----
        
    else if ( lsame(x_ext_type, 'p') ) then
        
        !$omp parallel do
        do iy = 1, my
            
            q(1-mbc:0,     iy) = q(mx-mbc+1:mx, iy)
            q(mx+1:mx+mbc, iy) = q(1:mbc,       iy)
        
        end do
    
    !---- Catch errors ----
    
    else
        
        print *, "Error (extend_to_ghosts): Provided invalid x extension type."
        stop
        
    end if
    
    !<=== Perform x-extension <===
        




    !===> Perform y-extension ===>
    
    !---- Even extension ----
    
    if ( lsame(y_ext_type, 'e') ) then
        
        do iy = 1-mbc, 0
            iy_target = 1-iy
            
            !$omp parallel do
            do ix = 1-mbc, mx+mbc
                q(ix, iy) = q(ix, iy_target)
            end do
        end do
        
        do iy = my+1, my+mbc
            iy_target = my - (iy-my) + 1
            
            !$omp parallel do
            do ix = 1-mbc, mx+mbc
                q(ix, iy) = q(ix, iy_target)
            end do
        end do
        
    !---- Odd extension ----
    
    else if ( lsame(y_ext_type, 'o') ) then
        
        do iy = 1-mbc, 0
            iy_target = 1-iy

            !$omp parallel do
            do ix = 1-mbc, mx+mbc
                q(ix, iy) = -q(ix, iy_target)
            end do
        end do
        
        do iy = my+1, my+mbc
            iy_target = my - (iy-my) + 1

            !$omp parallel do
            do ix = 1-mbc, mx+mbc
                q(ix, iy) = -q(ix, iy_target)
            end do
        end do
        
    !---- Periodic extension ----
    
    else if ( lsame(y_ext_type, 'p') ) then
        
        do iy = 1-mbc, 0
            iy_target = iy + my

            !$omp parallel do
            do ix = 1-mbc, mx+mbc
                q(ix, iy) = q(ix, iy_target)
            end do
        end do
        
        do iy = my+1, my+mbc
            iy_target = iy - my

            !$omp parallel do
            do ix = 1-mbc, mx+mbc
                q(ix, iy) = q(ix, iy_target)
            end do
        end do
        
    !---- Catch errors ----
    
    else
        
        print *, "Error (extend_to_ghosts): Provided invalid y extension type."
        stop
        
    end if
    
    !<=== y-extension <===
    

    
end subroutine extend_to_ghosts