subroutine laplacian(mx, my, mbc, dx, dy,  &
                     q,  &
                     q_laplacian)

!-------------------------------------------------------------
! Calculate Laplacian of q at cell centers.  Ghost cells
! should be filled to enforce the desired boundary conditions
! prior to calling this routine.
!-------------------------------------------------------------


    !$ use omp_lib
    implicit none

    !---> Variables --->
    ! Inputs 
    integer, intent(in) :: mx, my, mbc
    double precision, intent(in) :: q(1-mbc:mx+mbc,1-mbc:my+mbc), dx, dy

    ! Outputs 
    double precision, intent(out) :: q_laplacian(1-mbc:mx+mbc,1-mbc:my+mbc)
    !$ common /omp_block/ nthreads

    ! Locals 
    integer :: ix, iy
    !$ integer :: nthreads
    !<--- Variables <---


    !$ call omp_set_num_threads(nthreads)
    
    !$omp parallel do private(ix)
    do iy = 2-mbc,my+mbc-1
        do ix = 2-mbc,mx+mbc-1
            q_laplacian(ix,iy) = (q(ix-1,iy) - 2.d0*q(ix,iy) + q(ix+1,iy)) / dx**2
            q_laplacian(ix,iy) = q_laplacian(ix,iy) + (q(ix,iy-1) - 2.d0*q(ix,iy) +  &
                                 q(ix,iy+1)) / dy**2
        end do
    end do
    


end subroutine laplacian