!-------------------------------------------------------------------
! These subroutines are based on the multilayer equation of state,
!       sigma(gamma) = 1/(1+mu*gamma)**3.
! Note that only the derivatives are used in the code; sigma itself
! never needs to be computed directly.
!
! The routines are designed to compute quantities on 1-dimensional
! slices.  Pointwise evaluation is inefficient, and produces a
! noticeable slowdown in the program.
!
! It might be better to make these into 'elemental' subroutines,
! but I haven't had a chance to play with that yet.
!-------------------------------------------------------------------


subroutine surface_tension_d1(tension_d1, mu, mx)
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: mu
    double precision, intent(inout) :: tension_d1(mx)
    integer :: i
    
    do i=1,mx
        tension_d1(i) = -3.d0*mu*(1.d0+mu*tension_d1(i))**(-4)
    end do
end subroutine surface_tension_d1


subroutine surface_tension_d2(tension_d2, mu, mx)
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: mu
    double precision, intent(inout) :: tension_d2(mx)
    integer :: i
    
    do i=1,mx
        tension_d2(i) = 12.d0*mu**2 * (1.d0+mu*tension_d2(i))**(-5)
    end do
end subroutine surface_tension_d2


double precision function surface_tension_d1_pw(g)
    implicit none
    double precision :: g
    double precision :: beta, kappa, mu, delta
    common /param_block/ beta, kappa, mu, delta
    surface_tension_d1_pw = -3.d0 * mu * (1.d0 + mu*g)**(-4)
end function surface_tension_d1_pw

double precision function surface_tension_d2_pw(g)
    implicit none
    double precision :: g
    double precision :: beta, kappa, mu, delta
    common /param_block/ beta, kappa, mu, delta
    surface_tension_d2_pw = 12.d0 * mu**2 * (1.d0 + mu*g)**(-5)
end function surface_tension_d2_pw