
from setrun import setrun
from setplot import setplot
from pyclaw.runclaw import runclaw
from pyclaw.plotters.plotclaw import plotclaw

from numpy import *
import pylab, os



def timeRefinementTest(velocity, diffusivity, dt_values, split_order):
    
    n_runs = len(dt_values)
    
    rundata = setrun()
    rundata.clawdata.nout = 1
    rundata.clawdata.dt_variable = 0


    #---- Set problem parameters ----
    rundata.probdata.velocity    = velocity
    rundata.probdata.diffusivity = diffusivity
    rundata.probdata.split_order = split_order


    #---- Baseline discretization parameters ----
    rundata.clawdata.xlower =  1.0
    rundata.clawdata.xupper =  2.0


    #---- dt refinement parameters ----
    t_final_target = 0.5


    
    
    mx_base = 0
    

    
    if abs(velocity)<1e-6:
        max_velocity = 1.0
    else:
        max_velocity = velocity
    
    dx_base = max_velocity*dt_values[0] / .95
    mx_base = int( (rundata.clawdata.xupper - rundata.clawdata.xlower) / dx_base )

        
    mx_values = zeros(n_runs, dtype='int')
    dx_values = zeros(n_runs)
    for i in range(n_runs):
        mx_values[i] = int(mx_base * dt_values[0] / dt_values[i])
        dx_values[i] = (rundata.clawdata.xupper - rundata.clawdata.xlower) / float(mx_values[i])
    print mx_values





    #===> Compute error for each dt ===>
    error_values   = zeros(n_runs)

    for i_run in range(n_runs):

        #---- Discretization parameters for this run ----
        dt      = dt_values[i_run]
        t_final = round(t_final_target/dt)*dt
        mx      = mx_values[i_run]
        
        #---- Write parameters to clawdata ----
        rundata.clawdata.dt_initial = dt
        rundata.clawdata.tfinal     = t_final
        rundata.clawdata.mx         = mx
    
        rundata.write()

        #---- Execute the run ----
        runclaw(xclawcmd = "xclaw", outdir="_output")
    
        #---- Read the final numerical error ----
        f = open("_output/error_value.txt", "r")
        error_values[i_run] = float(f.readline())
        f.close()
    
    #<=== Compute error for each dt <===
    
    return error_values, dx_values



dt_values = logspace(-1,-3,21)

lw_cn_errors, dx_values = timeRefinementTest(1.0, 1e-1, dt_values, 1)
cn_lw_errors, dx_values = timeRefinementTest(1.0, 1e-1, dt_values, 2)

#===> Produce error plots ===>

plot_parameters = {'backend': 'eps',
          'lines.linewidth': 4,
          'lines.markersize': 10,
          'text.fontsize': 10,
          'font.size'    : 10,
          'legend.fontsize': 10,
          'xtick.labelsize': 10,
          'ytick.labelsize': 10,
          'axes.labelsize':  14,
          'figure.subplot.left':   0.18,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.15,
          'figure.subplot.top':    0.9}
          
pylab.rcParams.update(plot_parameters)


pylab.figure(figsize=(4,3), facecolor='w', edgecolor='k')


dt_squared = dt_values**2
pylab.loglog(dt_values, dt_squared, 'k-', label="$\Delta t^2$")
pylab.loglog(dt_values, lw_cn_errors, 'k.', label="LW/CN errors")
pylab.loglog(dt_values, cn_lw_errors, 'k+', label="CN/LW errors")


pylab.xlabel('$\Delta t$')

pylab.xlim(1e-3/1.3,2e-1)

pylab.legend(loc='lower right')


#<=== Produce error plots <===





