clear all
clc

nx = 100;
x_low = 0.0;
x_high = 1.0;
dx = (x_high - x_low) / nx;

x = (x_low+dx/2:dx:x_high-dx/2)';
coordinate = @(ix) x_low + (ix-.5)*dx;



q = zeros(nx,1);


diffusivity = 5e-2;
courant = 20;
dt = courant * dx^2 / diffusivity;

t_final = 1.0;

t0 = .3;
q_true = @(x,t) 1/sqrt(4*pi*diffusivity*(t+t0)) * exp(-(x-.5).^2 / (4*diffusivity*(t+t0)) );



t = 0.0;
q = q_true(x, 0);


A = backwardEulerMatrix(courant/2, nx);


plot(x, q)
pause(1)


while (abs(t < t_final) > 1e-8)
    if (t + dt > t_final)
        dt = t_final - t;
    end
    
    q_left_old  = q_true( coordinate(0),    t );
    q_right_old = q_true( coordinate(nx+1), t );    
    
    q_left_new  = q_true( coordinate(0),    t+dt );
    q_right_new = q_true( coordinate(nx+1), t+dt );
    

    %---- Estimate the mid left boundary value ----
%     q_left_temp = (q_left_new - courant*(-5*q(1) + 4*q(2) - q(3))) / (1 + 2*courant);
%     q_left_mid = q_left_temp + courant/2*(2*q_left_temp - 5*q(1) + 4*q(2) - q(3));

    q_left_mid = q_left_old + courant/2*(2*q_left_old - 5*q(1) + 4*q(2) - q(3));
    
%     q_left_mid = q_true( coordinate(0), t+dt/2 );
 
    q_right_mid = q_true( coordinate(nx+1), t+dt/2 );


    %---- First half-step ----
    
    rhs_1_base = q(1) ...
        + courant^2 * (.25 - (.5*(1+.5*courant*2))/(1+courant*2)) * (-5*q(1) + 4*q(2) - q(3)) ...
        + .5*courant*(1+.5*courant*2)/(1+courant*2) * q_left_new;
    
    rhs     = q;    
    rhs(1)  = rhs(1)  + courant/2 * q_left_mid;
    rhs(nx) = rhs(nx) + courant/2 * q_right_mid;
    q = A\rhs;
    
    rhs_1_test = (1+courant)*q(1) - .5*courant*q(2);
    
    fprintf(1, '%f  %f\n', rhs_1_base, rhs_1_test)
    
    
    %---- Second half-step ----
    rhs     = q;    
    rhs(1)  = rhs(1)  + courant/2 * q_left_new;
    rhs(nx) = rhs(nx) + courant/2 * q_right_new;
    q = A\rhs;
    
    
    %---- Update time and plot ----
    t = t + dt;
    plot(x, q-q_true(x,t), '-o')
    pause(.1)

    
end

