clear all
clc


%---- Problem parameters ----
diffusivity = 5e-2;
t0 = .3;
q_true = @(x,t) 1/sqrt(4*pi*diffusivity*(t+t0)) * exp(-(x-.5).^2 / (4*diffusivity*(t+t0)) );


%---- Spatial parameters ----
nx     = 100;
x_low  = 0.0;
x_high = 1.0;
dx     = (x_high - x_low) / nx;
x      = (x_low+dx/2:dx:x_high-dx/2)';
coordinate = @(ix) x_low + (ix-.5)*dx;



%---- Time parameters ----
courant = 10;
dt = courant * dx^2 / diffusivity;
t_final = .25;



%---- Create operators ----
BE_half = backwardEulerMatrix(courant/2, nx);
FE_half = forwardEulerMatrix(courant/2, nx);



%---- Initialize solution ----
q = zeros(nx,1);
t = 0.0;
q = q_true(x, 0);
plot(x, q)
pause(1)



%===> Time-stepping ===>

while (abs(t < t_final) > 1e-8)
    
    %---- Adjust dt if necessary ----
    if (t + dt > t_final)
        dt = t_final - t;
    end
    
    
    %---- Old boundary data ----
    q_left_old = q_true( coordinate(0), t );
    q_right_old = q_true( coordinate(nx+1), t );
    
    
    %---- New boundary data ----
    q_left_new = q_true( coordinate(0), t+dt );
    q_right_new = q_true( coordinate(nx+1), t+dt );


    %---- Forward Euler half-step ----
    q = FE_half*q;
    q(1)  = q(1)  + courant*q_left_old;
    q(nx) = q(nx) + courant*q_right_old;
    
    
    %---- Backwar Euler half-step ----
    rhs     = q;    
    rhs(1)  = courant * q_left_new;
    rhs(nx) = courant * q_right_new;
    q = BE_half\rhs;
    
    
    %---- Update ----
    t = t + dt;
    plot(x, q)
    pause(.1)
    
    
end

%<=== Time-stepping <===