clear all
clc


%---- Problem parameters ----
diffusivity = 5e-2;
t0 = .3;
q_true = @(x,t) 1/sqrt(4*pi*diffusivity*(t+t0)) * exp(-(x-.1).^2 / (4*diffusivity*(t+t0)) );


%---- Spatial parameters ----
nx     = 100;
x_low  = 0.0;
x_high = 1.0;
dx     = (x_high - x_low) / nx;
x      = (x_low+dx/2:dx:x_high-dx/2)';
coordinate = @(ix) x_low + (ix-.5)*dx;



%---- Time parameters ----
courant = 20;
dt = courant * dx^2 / diffusivity;
t_final = 1.0;



%---- Create operators ----
BE_quarter = backwardEulerMatrix(courant/4, nx);
FE_quarter = forwardEulerMatrix(courant/4, nx);



%---- Initialize solution ----
q = zeros(nx,1);
t = 0.0;
q = q_true(x, 0);
plot(x, q - q_true(x,0) )
pause(1)



%===> Time-stepping ===>

while (abs(t < t_final) > 1e-8)
    
    %---- Adjust dt if necessary ----
    if (t + dt > t_final)
        dt = t_final - t;
    end
    
    
    %---- Old boundary data ----
    q_left_old = q_true( coordinate(0), t );
    q_right_old = q_true( coordinate(nx+1), t );
    
    
    %---- New boundary data ----
    q_left_new = q_true( coordinate(0), t+dt );
    q_right_new = q_true( coordinate(nx+1), t+dt );
    
    
    %---- Middle boundary data ----
    q_left_temp = (q_left_new ...
                    - courant*(-5*q(1) + 4*q(2) - q(3))) ...
                    / (1 + 2*courant);


    q_left_temp2 = (q_left_new ...
                    - courant*(-5*q(1) + 4*q(2) - q(3)) ...
                    - courant^2/2*(-14*q(1) + 26*q(2) - 24*q(3) + 11*q(4) - 2*q(5))) ...
                    / (1 + 2*courant + 3*courant^2/2);


%     q_left_mid = q_true( coordinate(0), t+dt/2 );
%     q_left_mid = q_left_old + courant/2*(2*q_left_old - 5*q(1) + 4*q(2) - q(3));
    q_left_mid = q_left_temp + courant/2*(2*q_left_temp - 5*q(1) + 4*q(2) - q(3));
%     q_left_mid = q_left_temp2 ...
%                   + courant/2 * (2*q_left_temp2 - 5*q(1) + 4*q(2) - q(3)) ...
%                   + courant^2/4 * (3*q_left_temp2 - 14*q(1) + 26*q(2) - 24*q(3) + 11*q(4) - 2*q(5));
%     q_left_mid = q_left_new - courant/2 * (2*q_left_old - 5*q(1) + 4*q(2) - q(3));
%     q_left_mid = (q_left_old + q_left_new)/2;

    
%     q_left_mid = q_left_old + courant/2*(2*q_left_old - 5*q(1) + 4*q(2) - q(3));
%     q_left_mid = q_left_new - courant/2 * (2*q_left_old - 5*q(1) + 4*q(2) - q(3));
%     q_left_mid  = q_true( coordinate(0),    t+dt/2 );
    q_right_mid = q_true( coordinate(nx+1), t+dt/2 );
    
    fprintf(1, '%f  %f\n', q_left_mid, q_true(coordinate(0), t+dt/2))


%     q_left_old = q_left_temp;

    %---- (1) Forward Euler half-step ----
    q = FE_quarter*q;
    q(1)  = q(1)  + courant/4*q_left_old;
    q(nx) = q(nx) + courant/4*q_right_old;
%     plot(x, q - q_true(x,t+dt/4), '-o' )
%     pause()

    
    
    %---- (1) Backward Euler half-step ----
    rhs     = q;    
    rhs(1)  = rhs(1)  + courant/4 * q_left_mid;
    rhs(nx) = rhs(nx) + courant/4 * q_right_mid;
    q = BE_quarter\rhs;
%     plot(x, q - q_true(x,t+dt/2), '-o' )
%     pause()

    
    
%     q_left_mid = (q_left_new - courant/2*(-5*q(1) + 4*q(2) - q(3))) / (1
%     + 2*courant/2);
    
    %---- (2) Forward Euler half-step ----
    q = FE_quarter*q;
    q(1)  = q(1)  + courant/4*q_left_mid;
    q(nx) = q(nx) + courant/4*q_right_mid;
%     plot(x, q - q_true(x,t+3*dt/4), '-o' )
%     pause()

    
    %---- (2) Backward Euler half-step ----
    rhs     = q;    
    rhs(1)  = rhs(1)  + courant/4 * q_left_new;
    rhs(nx) = rhs(nx) + courant/4 * q_right_new;
    q = BE_quarter\rhs;
    
    
    
    %---- Update ----
    t = t + dt;
    plot(x, q - q_true(x,t), '-o' )
    pause()
    
    
end

%<=== Time-stepping <===