
from setrun import setrun
from setplot import setplot
from pyclaw.runclaw import runclaw
from pyclaw.plotters.plotclaw import plotclaw

from numpy import *
import pylab, os



def courantStabilityTest(mx, diffusivity, courant_values, split_order):
    
    n_runs = len(courant_values)
    
    rundata = setrun()
    rundata.clawdata.nout = 1
    rundata.clawdata.dt_variable = 0


    #---- Set problem parameters ----
    rundata.probdata.diffusivity = diffusivity
    rundata.probdata.split_order = split_order


    #---- Baseline discretization parameters ----
    rundata.clawdata.xlower =  1.0
    rundata.clawdata.xupper =  2.0
    rundata.clawdata.mx     =  mx
    dx = 1.0 / float(mx)
    
    

    #---- dt refinement parameters ----
    t_final_target = 0.5
    dt = 0.01    
    t_final = round(t_final_target/dt)*dt

    rundata.clawdata.dt_initial = dt
    rundata.clawdata.tfinal     = t_final




    #===> Compute error for each dt ===>
    error_values   = zeros(n_runs)

    for i_run in range(n_runs):

        #---- Discretization parameters for this run ----
        velocity = courant_values[i_run]*dx/dt
        rundata.probdata.velocity = velocity
        rundata.write()
    

        #---- Execute the run ----
        runclaw(xclawcmd = "xclaw", outdir="_output")
    
        #---- Read the final numerical error ----
        f = open("_output/error_value.txt", "r")
        error_values[i_run] = float(f.readline())
        f.close()
    
    #<=== Compute error for each dt <===
    
    return error_values



courant_values = linspace(0.5,1.2,41)

lw_cn_errors = courantStabilityTest(100, 1e-1, courant_values, 1)
cn_lw_errors = courantStabilityTest(100, 1e-1, courant_values, 2)

#===> Produce error plots ===>

plot_parameters = {'backend': 'eps',
          'lines.linewidth': 4,
          'lines.markersize': 10,
          'text.fontsize': 10,
          'font.size'    : 10,
          'legend.fontsize': 10,
          'xtick.labelsize': 10,
          'ytick.labelsize': 10,
          'axes.labelsize':  14,
          'figure.subplot.left':   0.18,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.15,
          'figure.subplot.top':    0.9}
          
pylab.rcParams.update(plot_parameters)


pylab.figure(figsize=(4,3), facecolor='w', edgecolor='k')


pylab.semilogy( courant_values, lw_cn_errors, 'k.', label='LW/CN errors')
pylab.semilogy( courant_values, lw_cn_errors, 'k+', label='CN/LW errors')

pylab.xlabel('$\\nu$')
# pylab.ylabel('$\infty$-norm error at $t_{\\rm final}$')


pylab.legend(loc='upper left')
pylab.xlim(.5,1.22)


#<=== Produce error plots <===





