subroutine lax_wendroff_1d( q, velocity, dx, dt, mx )
    
    implicit none
    
    !---- Variables ----

    integer, intent(in) :: mx
    double precision, intent(in) :: velocity, dx, dt
    double precision, intent(inout) :: q(0:mx+1)

    integer :: ix
    double precision :: q_new(mx), courant
    

    courant = velocity*dt/dx

!     !---- Extrapolate the downstream boundary value ----
! 
!     if (velocity > 0.d0) then
!         q(mx+1) = q(mx)
!     else
!         q(0) = q(1)
!     end if
    
    
    
    !---- Update ----
    
    do ix = 1,mx

        q_new(ix) = q(ix) - courant/2.0 * (q(ix+1) - q(ix-1))  &
                          + courant**2/2.d0 * (q(ix+1) - 2*q(ix) + q(ix-1))

    end do
    
    q_new(mx) = q(mx) - courant*(1.5d0*q(mx) - 2.d0*q(mx-1) + .5d0*q(mx-2))  &
                      + courant**2/2.d0 * (q(mx) - 2.d0*q(mx-1) + q(mx-2))
    
    
    q(1:mx) = q_new


    
end subroutine lax_wendroff_1d



subroutine upwind_1d( q, velocity, dx, dt, mx )
    
    implicit none
    
    !==== Variables ====
    !---- Input ----
    integer, intent(in) :: mx
    double precision, intent(in) :: velocity, dx, dt
    
    !---- Input/output ----
    double precision, intent(inout) :: q(0:mx+1)
    
    !---- Locals ----
    integer :: ix
    double precision :: q_new(mx)
    

    
    if (velocity >= 0) then
        do ix = 1,mx
            q_new(ix) = q(ix) - velocity*dt/dx * (q(ix) - q(ix-1))
        end do
    else
        do ix = 1,mx
            q_new(ix) = q(ix) - velocity*dt/dx * (q(ix+1) - q(ix))
        end do
    end if
            
    q(1:mx) = q_new


    
end subroutine upwind_1d




subroutine upwind_1d_split( q, velocity, x_low, dx, t, dt, mx )
    
    implicit none
    
    !==== Variables ====
    !---- Input ----
    integer, intent(in) :: mx
    double precision, intent(in) :: velocity, t, dt, x_low, dx
    
    !---- Input/output ----
    double precision, intent(inout) :: q(0:mx+1)
    
    !---- Locals ----
    integer :: ix
    double precision :: coordinate, q_true
    
    
    !==== Statement functions ====
    coordinate(ix) = x_low + (ix-.5d0)*dx
    

    !---- Advance first half-step ----
    q(0) = q_true( coordinate(0), t )
    call upwind_1d( q, velocity, dx, dt/2, mx )
    
    
    !---- Estimate new boundary condition ----
!     q(0) = q(0) - velocity * dt / dx * (q(1) - q(0))
    
    
    !---- Advance second half-step ----
    call upwind_1d( q, velocity, dx, dt/2, mx )


    
end subroutine upwind_1d_split




subroutine beam_warming_1d( q, velocity, dx, dt, mx )

    implicit none
    
    !==== Variables ====
    !---- Input ----
    integer, intent(in) :: mx
    double precision, intent(in) :: velocity, dx, dt
    
    !---- Input/output ----
    double precision, intent(inout) :: q(-1:mx)
    
    !---- Locals ----
    integer :: ix
    double precision :: q_new(mx)
    
    
    do ix = 1,mx
        q_new(ix) = q(ix) - velocity*dt * (3*q(ix) - 4*q(ix-1) + q(ix-2)) / (2*dx)          &
                          + (velocity*dt)**2/2 * (q(ix) - 2*q(ix-1) + q(ix-2)) / dx**2 
    end do
    
    q(1:mx) = q_new
    
end subroutine beam_warming_1d