subroutine advdiff_backward_estimate ( old_bcs, q, new_bcs, velocity, diffusivity, dx, dt, mx )
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(mx), new_bcs(2), velocity, diffusivity, dx, dt
    double precision, intent(out) :: old_bcs(2)
    double precision :: courant, stab_rat
    
    courant  = velocity*dt/dx
    stab_rat = diffusivity*dt/dx**2
    
    old_bcs(1) = ( new_bcs(1) + courant*q(1)               &
                   - stab_rat*(-5*q(1) + 4*q(2) - q(3)) )     &
                 / ( 1.d0 + courant + 2*stab_rat )
    
    old_bcs(2) = ( new_bcs(2) - courant*q(mx)                    &
                   - stab_rat*(-q(mx-2) + 4*q(mx-1) - 5*q(mx)) ) &
                / ( 1.d0 - courant + 2*stab_rat )


! For testing order 1 estimate of second derivative
!
!     old_bcs(1) = ( new_bcs(1) + courant*q(1)        &
!                    - stab_rat*(-2*q(1) + q(2)) )    &
!                  / ( 1.d0 + courant + stab_rat )
! 
!     old_bcs(2) = ( new_bcs(2) - courant*q(mx)       &
!                    - stab_rat*(q(mx-1) - 2*q(mx)) ) &
!                 / ( 1.d0 - courant + stab_rat )

                                        
end subroutine advdiff_backward_estimate



subroutine advection_forward_estimate ( new_bcs, q, old_bcs, velocity, dx, dt, mx )
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(mx), old_bcs(2), velocity, dx, dt
    double precision, intent(out) :: new_bcs(2)
    double precision :: courant
    
    courant = velocity*dt/dx
    
    new_bcs(1) = old_bcs(1) - courant * (q(1) - old_bcs(1))
    new_bcs(2) = old_bcs(2) - courant * (old_bcs(2) - q(mx))
    
end subroutine advection_forward_estimate



subroutine diffusion_forward_estimate ( new_bcs, q, old_bcs, diffusivity, dx, dt, mx )
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(mx), old_bcs(2), diffusivity, dx, dt
    double precision, intent(out) :: new_bcs(2)
    double precision :: stab_rat
    
    stab_rat = diffusivity*dt/dx**2
    
    new_bcs(1) = old_bcs(1) + stab_rat * (2*old_bcs(1) - 5*q(1) + 4*q(2) - q(3))
    new_bcs(2) = old_bcs(2) + stab_rat * (-q(mx-2) + 4*q(mx-1) - 5*q(mx) + 2*old_bcs(2))


! For testing order 1 estimate of second derivative
!
!     new_bcs(1) = old_bcs(1) + stab_rat * (old_bcs(1) - 2*q(1) + q(2))
!     new_bcs(2) = old_bcs(2) + stab_rat * (q(mx-1) - 2*q(mx) + old_bcs(2))

    
end subroutine diffusion_forward_estimate
