from numpy import *


def BackwardEulerMatrix(courant, n):

    M = zeros( (n,n) )
    
    M[0,0] = 1. + 2*courant
    M[0,1] = -courant

    for i in range(1,n-1):
    
        M[i,i-1] = -courant
        M[i,i]   = 1. + 2*courant
        M[i,i+1] = -courant
    
    M[n-1,n-2] = -courant
    M[n-1,n-1]   = 1. + 2*courant


    return M




def ForwardEuler( q, diffusivity, dx, dt ):

    n = size(q,0)
    
    q_new = q.copy()
    
    for i in range(1,n-1):
        q_new[i] = q[i] + diffusivity*dt/dx**2 * (q[i+1] - 2*q[i] + q[i-1])
    
    return q_new




def BackwardEuler( q, diffusivity, (q_left, q_right), dx, dt ):
    
    n = size(q,0)
    courant = diffusivity*dt/dx**2

    BE = BackwardEulerMatrix( courant, n-2 )
    
    q_new = q
    q_new[1]  += courant*q_left
    q_new[-1] += courant*q_right
    
    q_new[1:-1] = linalg.solve(BE, q_new[1:-1])
    q_new[0] = q_left
    q_new[-1] = q_right
    
    return q_new



def CrankNicolson( q, diffusivity, new_bcs, dx, dt ):
        
    q_new = ForwardEuler( q, diffusivity, dx, dt/2 )
    
    q_new = BackwardEuler( q_new, diffusivity, new_bcs, dx, dt/2 )
    
    return q_new



def TGA( q, diffusivity, new_bcs, dx, dt ):

    a = 2. - sqrt(2.)
    r1 = 1.-a
    r2 = (2.*a-1.) / (a + sqrt(a**2 - 4.*a + 2.))
    r3 = (2.*a-1.) / (a - sqrt(a**2 - 4.*a + 2.))

    mid_bcs = ( q[0] + (r1+r2)*(new_bcs[0]-q[0]),  q[-1] + (r1+r2)*(new_bcs[1]-q[-1]) )
    
    q_new = ForwardEuler( q, diffusivity, dx, r1*dt )
    q_new = BackwardEuler( q_new, diffusivity, mid_bcs, dx, r2*dt )
    q_new = BackwardEuler( q_new, diffusivity, new_bcs, dx, r3*dt )

    return q_new