from numpy import *
import Diffusion1D
import pylab, os


def q_true(x,t):
    return exp(-pi**2/4*t) * cos(pi/2*x)


diffusivity = 1.

nx = 40
dx = 1./float(nx)
x = linspace(0, 1, nx+2)

dt = 0.01

q1 = q_true(x, 0.)
# q2 = zeros(nx)



q_left = q_true(0, dt)
q_right = q_true(1, dt)


#---- Step q1 with TGA ----
q1 = q_true(x,0)
q1[0] = 0.
q1 = Diffusion1D.TGA( q1, diffusivity, (0.,0.), dx, dt )


#---- Step q2 with Crank-Nicolson ----
q2 = zeros(nx+2)
q2[0] = q_true(0.,0.)
q2 = Diffusion1D.CrankNicolson( q2, diffusivity, (q_left, 0.), dx, dt )




plot_parameters = {'backend': 'eps',
          'lines.linewidth': 3,
          'text.fontsize': 8,
          'legend.fontsize': 8,
          'xtick.labelsize': 8,
          'ytick.labelsize': 8,
          'axes.labelsize':  10,
          'figure.subplot.left':   0.15,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.15,
          'figure.subplot.top':    0.95}
          
pylab.rcParams.update(plot_parameters)


fig = pylab.figure(figsize=(3,2), facecolor='w', edgecolor='k')

# axes1 = fig.add_subplot(1,2,1)
# # axes1.plot(x, q1+q2 - q_true(x, dt), 'k', label='True solution', linewidth=1.5)
# axes1.plot(x, q1+q2-q_true(x,dt), 'r-o', label='Approximation to $q(x,\Delta t)$')
# axes1.set_ylim(-.1,1.2)
# axes1.legend()
# 
# 
# axes2 = fig.add_subplot(1,2,2)
# axes2.plot(x, q1, 'b-o', label='Approximation to $q_1(x,\Delta t)$')
# axes2.plot(x, q2, 'r-o', label='Approximation to $q_2(x,\Delta t)$')
# axes2.set_ylim(-.1,1.2)
# axes2.legend()

ax = fig.add_subplot(1,1,1)
ax.plot(x, q1, 'k-.',  label='$Q_1(\Delta t)$', linewidth=2, markersize=2)
ax.plot(x, q2, 'k:', label='$Q_2(\Delta t)$', linewidth=2, markersize=2)
ax.plot(x, q1+q2-q_true(x,dt), 'k-', label='$Q_1(\Delta t) + Q_2(\Delta t) - q(x,\Delta t)$')
ax.set_xlabel('$x$')
ax.set_ylabel('$q$')
ax.set_ylim(-.1,1.5)
ax.legend()

fig.savefig('BoundaryMismatch.eps')
os.system('epstopdf BoundaryMismatch.eps')