class SimulationData:
    """
    This class holds a set of data for a simple 3D numerical
    solution of the diffusion equation.
    """
    
    def __init__(self):

        self.x_low       = (-1.0,-1.0,-1.0)
        self.x_high      = (1.0,1.0,1.0)
        self.t_limits    = (0.0,0.5)
        self.dt          = 1e-1
        self.n_cells     = (40,40,40)
        self.diffusivity = 5e-2
        
        self.data_file_name = "SimulationData.txt"
        

    def writeDataLine(self, data_file, data, comment=""):
        """ Writes scalar or tuple data to a line of the provided outpt file. """
        
        if type(data)==tuple:        
            for i in range( len(data) ):
                data_file.write( str(data[i]) + " " )            
        else:
            data_file.write( str(data) + " " )
        
        if len(comment) > 0:
            data_file.write( "     " + comment )
        data_file.write( "\n" )
    
        
        
    def writeDataFile(self):
        """ Writes the full data file. """

        data_file = open(self.data_file_name, "w")

        self.writeDataLine( data_file, self.x_low,      "x_low" )
        self.writeDataLine( data_file, self.x_high,     "x_high" )
        self.writeDataLine( data_file, self.t_limits,   "t-limits" )
        self.writeDataLine( data_file, self.dt,         "dt" )
        self.writeDataLine( data_file, self.n_cells,    "n_cells" )
        self.writeDataLine( data_file, self.diffusivity, "diffusivity" )

        data_file.close()
        
