subroutine cn_2d_split( q, x_new_bcs, y_new_bcs, diffusivity, dx, dy, dt, mx, my )

    implicit none
    
    !===> Variables ===>
        
    !---- Inputs ----
    integer, intent(in) :: mx, my
    double precision, intent(in) :: x_new_bcs(0:my+1,2), y_new_bcs(0:mx+1,2)
    double precision, intent(in) :: diffusivity, dx, dy, dt
    
    !---- Input/output ----
    double precision :: q(0:mx+1, 0:my+1)
    
    !---- Locals ----
    integer :: ix, iy, iend
    double precision :: x_mid_bcs(0:my+1,2)
    
    !<=== Variables <===
    

    
    !---- Estimate left/right BCs at mid-step; includes corners ----

    do iend = 1,2
        call backward_estimate_cn( x_mid_bcs(:,iend), x_new_bcs(:,iend), diffusivity, dy, dt, my )
    end do
    

    !---- Horizontal problems; includes bottom/top boundaries ----
    
    do iy = 0, my+1
        call crank_nicolson_1d( q(0:mx+1,iy), x_mid_bcs(iy,1:2), diffusivity, dx, dt, mx )
    end do
    
    

    !---- Vertical problems ----
    
    do ix = 1, mx
        call crank_nicolson_1d( q(ix,0:my+1), y_new_bcs(ix,1:2), diffusivity, dy, dt, my )
    end do



end subroutine cn_2d_split




subroutine backward_estimate( q_back, q, diffusivity, dx, dt, mx )

    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(0:mx+1), diffusivity, dx, dt
    double precision, intent(out) :: q_back(0:mx+1)
    double precision :: diff_ratio
    integer :: i
    
    
    diff_ratio = diffusivity*dt/dx**2
    
    q_back(0) = q(0) - diff_ratio * (2*q(0) - 5*q(1) + 4*q(2) - q(3))
    
    do i = 1, mx
        q_back(i) = q(i) - diff_ratio * (q(i-1) - 2*q(i) + q(i+1))
    end do
    
    q_back(mx+1) = q(mx+1) - diff_ratio * (-q(mx-2) + 4*q(mx-1) - 5*q(mx) + 2*q(mx+1))
    
end subroutine backward_estimate




subroutine backward_estimate_cn( q_back, q, diffusivity, dx, dt, mx )

    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(0:mx+1), diffusivity, dx, dt
    double precision, intent(out) :: q_back(0:mx+1)
    double precision :: diff_ratio, back_bcs(2)    


    diff_ratio = diffusivity*dt/dx**2
    
    back_bcs(1) = q(0)    - diff_ratio * (2*q(0) - 5*q(1) + 4*q(2) - q(3))    
    back_bcs(2) = q(mx+1) - diff_ratio * (-q(mx-2) + 4*q(mx-1) - 5*q(mx) + 2*q(mx+1))
    
    q_back = q
    
    call crank_nicolson_1d( q_back, back_bcs, diffusivity, dx, -dt, mx )
    
end subroutine backward_estimate_cn