subroutine cn_3d_split ( q, x_new_bcs, y_new_bcs, z_new_bcs, &
                         diffusivity, dx, dy, dz, dt, mx, my, mz )

    implicit none

    integer, intent(in) :: mx, my, mz
    double precision, intent(in) :: x_new_bcs(0:my+1,0:mz+1,2), &
                                    y_new_bcs(0:mx+1,0:mz+1,2), &
                                    z_new_bcs(0:mx+1,0:my+1,2)
    double precision, intent(in) :: diffusivity, dx, dy, dz, dt
    
    double precision, intent(inout) :: q(0:mx+1, 0:my+1, 0:mz+1)
    
    double precision :: x_mid_bcs(0:my+1,0:mz+1,2), &
                        y_mid_bcs(0:mx+1,0:mz+1,2)
    integer :: ix, iy, iz, iend
    
    

    !---- Backward estimate BCs across the z-sweep ----

    do iend = 1,2
        do iy = 0,my+1
            call backward_estimate_cn( x_mid_bcs(iy,:,iend), x_new_bcs(iy,:,iend), &
                                       diffusivity, dz, dt, mz )
        end do
    end do
    
    do iend = 1,2
        do ix = 0, mx+1
            call backward_estimate_cn( y_mid_bcs(ix,:,iend), y_new_bcs(ix,:,iend), &
                                       diffusivity, dz, dt, mz )
        end do
    end do
    
    
    
    !---- Solve 2D problem on each xy-plane, including bottom/top boundaries ----
    
    do iz = 0, mz+1
        call cn_2d_split( q(:,:,iz), x_mid_bcs(:,iz,1:2), y_mid_bcs(:,iz,1:2), &
                          diffusivity, dx, dy, dt, mx, my )
    end do
    
    
    !---- Diffuse in z-direction ----
    
    do iy = 1,my
        do ix = 1,mx
            call crank_nicolson_1d( q(ix,iy,:), z_new_bcs(ix,iy,1:2), diffusivity, dz, dt, mz )
        end do
    end do

    
end subroutine cn_3d_split