program dimension_splitting_3d
    
    implicit none
    
    !==== Variables ====
    integer :: n_cells(3), mx, my, mz
    double precision :: t, dt, t_limits(2)
    double precision :: x_low(3), x_high(3), dx(3)

    double precision :: diffusivity
    common /split_block/ diffusivity

    double precision, allocatable :: q(:,:,:), x_new_bcs(:,:,:), y_new_bcs(:,:,:), z_new_bcs(:,:,:)

    integer :: ix, iy, iz, i_dim
    double precision :: x, y, z, coordinate
    double precision :: q_true
    double precision :: error_value, new_error_value
    
    
    
    !---- Statement functions for coordinate values ----
    coordinate(i_dim,ix) = x_low(i_dim) + (ix-.5d0)*dx(i_dim)
    
    
    !===> Read simulation parameters from file ===>

    open(7, file="SimulationData.txt", status="old")

    read(7,*) x_low
    read(7,*) x_high
    read(7,*) t_limits

    read(7,*) dt
    read(7,*) n_cells
    
    read(7,*) diffusivity
    
    do i_dim = 1,3
        dx(i_dim) = (x_high(i_dim) - x_low(i_dim)) / n_cells(i_dim)
    end do

    close(7)
    
    
    mx = n_cells(1)
    my = n_cells(2)
    mz = n_cells(3)
    
    !<=== Read simulation parameters from file <===
    
    
    
    !===> Allocate and initialize solution ===>
    
    allocate( q(0:mx+1, 0:my+1, 0:mz+1) )
    allocate( x_new_bcs(0:my+1,0:mz+1,2) )
    allocate( y_new_bcs(0:mx+1,0:mz+1,2) )
    allocate( z_new_bcs(0:mx+1,0:my+1,2) )

    do iz = 1, mz
        z = coordinate(3,iz)
        
        do iy = 1, my
            y = coordinate(2,iy)
            
            do ix = 1, mx
                x = coordinate(1,ix)
                
                q(ix,iy,iz) = q_true( x, y, z, t_limits(1) )
            end do
        end do
    end do

    !<=== Allocate and initialize solution <===
    

    
    
    !===> Time-stepping ===>
    
    t = t_limits(1)
    
    do while ( t < t_limits(2)-1.d-8 )
        
        
        !---- Adjust time step if necessary ----
        if (t+dt > t_limits(2)) then
            dt = t_limits(2) - t
        end if
        
        
        !===> Fill boundary data ===>
        
        !---- x-bounds ----
        
        do iz = 0, mz+1
            z = coordinate(3,iz)
            
            do iy = 0, my+1
                y = coordinate(2,iy)
                
                x = coordinate(1,0)
                q(0,iy,iz)         = q_true( x, y, z, t )
                x_new_bcs(iy,iz,1) = q_true( x, y, z, t+dt )

                x = coordinate(1,mx+1)
                q(mx+1,iy,iz)      = q_true( x, y, z, t )
                x_new_bcs(iy,iz,2) = q_true( x, y, z, t+dt )

            end do
        end do
        
        
        
!         error_value = 0.d0
!         do iz = 0, mz+1
!             do iy = 0, my+1
!                 error_value = max( error_value, abs(x_new_bcs(iy,iz,1)))
!             end do
!         end do
!         print *, error_value
        
        
        !---- y-bounds ----
        
        do iz = 0, mz+1
            z = coordinate(3,iz)
            
            do ix = 0, mx+1
                x = coordinate(1,ix)
                
                y = coordinate(2,0)
                q(ix,0,iz)         = q_true( x, y, z, t )
                y_new_bcs(ix,iz,1) = q_true( x, y, z, t+dt )
                
                y = coordinate(2,my+1)
                q(ix,my+1,iz)      = q_true( x, y, z, t )
                y_new_bcs(ix,iz,2) = q_true( x, y, z, t+dt )
                
            end do
        end do
        
        
        !---- z bounds ----
        
        do iy = 0, my+1
            y = coordinate(2,iy)
            
            do ix = 0, mx+1
                x = coordinate(1,ix)
                
                z = coordinate(3,0)
                q(ix,iy,0)         = q_true( x, y, z, t )
                z_new_bcs(ix,iy,1) = q_true( x, y, z, t+dt )

                z = coordinate(3,mz+1)
                q(ix,iy,mz+1)      = q_true( x, y, z, t )
                z_new_bcs(ix,iy,2) = q_true( x, y, z, t+dt )
                
            end do
        end do
        
        !<=== Fill boundary data <===
        
        
        
        !---- Advance the solution ----
        call cn_3d_split( q, x_new_bcs, y_new_bcs, z_new_bcs,  &
                          diffusivity, dx(1), dx(2), dx(3), dt, mx, my, mz )
        
        
        !---- Update time ----
        t = t + dt

    end do
    
    !<=== Time-stepping <===


    
    
    
    
    
!     print *, q_true(x_low(1), 0.d0, 0.d0, t_limits(2))
    
    !---- Check numerical error ----
    
!     print *, t
    error_value = 0.d0
    
    do iz = 1, n_cells(3)
        z = coordinate(3,iz)
        
        do iy = 1, n_cells(2)
            y = coordinate(2,iy)
            
            do ix = 1, n_cells(1)
                x = coordinate(1,ix)
                
                new_error_value = abs(q(ix,iy,iz) - q_true(x,y,z,t))
                if (new_error_value > error_value) then
                    error_value = new_error_value
                    
!                     print *, ""
!                     print *, error_value
!                     print *, q(ix,iy,iz), q_true(x,y,z,t)
!                     print *, x, y, z
                end if
            end do  
        end do
    end do
    
        
    
    open( 7, file="NumericalError.txt" )
    write(7,*) error_value
    close(7)
    
    
    
    
end program dimension_splitting_3d


! subroutine scan(q, n_cells, n_ghosts)
!     implicit none
!     integer, intent(in) :: n_cells(3), n_ghosts
!     double precision, intent(in) :: q( 1-n_ghosts : n_cells(1)+n_ghosts,  &
!                                           1-n_ghosts : n_cells(2)+n_ghosts,  &
!                                           1-n_ghosts : n_cells(3)+n_ghosts )
!     integer :: ix, iy, iz
!     
!     do iz = 1, n_cells(3)
!         do iy = 1, n_cells(2)
!             do ix = 1, n_cells(1)
!                 if ( abs(q(ix,iy,iz)) > 1.d2 ) then
!                     print *, "Warning: q exceeds tolerance at ", ix, iy, iz
!                 end if
!             end do
!         end do
!     end do
!     
! end subroutine scan



function q_true ( x, y, z, t )
    implicit none
    double precision, intent(in) :: x, y, z, t
    double precision :: q_true
    double precision :: diffusivity, pi=acos(-1.d0), t0=.5d0, tau
    common /split_block/ diffusivity
    
    tau = t + t0
    
    q_true = 1.d0/(4*pi*diffusivity*tau)**(1.5d0) * exp( -(x**2+y**2+z**2) / (4*diffusivity*tau) )
    
!     q_true = exp( -3*pi**2 * diffusivity * t) * sin(pi*x) * sin(pi*y) * sin(pi*z)
    
end function q_true