from numpy import *





class FirstDerivativeOperator:
    
    dx = .1
    n  = 10
    extrap = 'none'
    
    def __init__(self, dx, n, extrap='none'):
        self.dx     = dx
        self.n      = n
        
        if (extrap=='low' or extrap=='high'):
            self.extrap = extrap
        else:
            self.extrap = 'none'
            
        self.buildInteriorOperator()
        self.buildBoundaryOperator()
            
    
    def buildInteriorOperator(self):
        
        #---- Initialize ----
        
        n  = self.n
        dx = self.dx
        D = zeros( (n,n) )
        
        
        #---- Low boundary ----

        if (self.extrap=='low'):
            D[0,0] = -.5/dx
                        
        D[0,1] = .5/dx


        #---- Interior ----
        
        for i in range(1,n-1):
            D[i,i-1] = -.5/dx
            D[i,i+1] =  .5/dx


        #---- High boundary ----

        D[-1,-2] = -.5/dx
        
        if (self.extrap=='high'):
            D[-1,-1] = .5/dx
        
        
        #---- Finish ----
    
        self.interior = D


    def buildBoundaryOperator(self):
        
        dx = self.dx
        B = zeros( (self.n,2) )

        if (self.extrap<>'low'):
            B[0,0]  = -.5/dx

        if (self.extrap<>'high'):
            B[-1,1] =  .5/dx

        self.boundary = B





class SecondDerivativeOperator:
    
    dx = .1
    n  = 10
    extrap = 'none'

    def __init__(self, dx, n, extrap='none'):
        self.dx     = dx
        self.n      = n
        
        if (extrap=='low' or extrap=='high'):
            self.extrap = extrap
        else:
            self.extrap = 'none'

        self.buildInteriorOperator()
        self.buildBoundaryOperator()


    def buildInteriorOperator(self):

        #---- Initialize ----

        n  = self.n
        dx = self.dx
        D = zeros( (n,n) )


        #---- Low boundary ----

        if (self.extrap=='low'):
            D[0,0] = -1./dx**2
        else:
            D[0,0] = -2./dx**2
        
        D[0,1] = 1./dx**2


        #---- Interior ----

        for i in range(1,n-1):
            D[i,i-1] =  1./dx**2
            D[i,i]   = -2./dx**2
            D[i,i+1] =  1./dx**2


        #---- High boundary ----

        D[-1,-2] = 1./dx**2
        
        if (self.extrap=='high'):
            D[-1,-1] = -1./dx**2
        else:
            D[-1,-1] = -2./dx**2
            

        #---- Finish ----

        self.interior = D



    def buildBoundaryOperator(self):

        dx = self.dx
        B = zeros( (self.n,2) )

        if (self.extrap<>'low'):
            B[0,0]  = 1./dx**2

        if (self.extrap<>'high'):
            B[-1,1] = 1./dx**2

        self.boundary = B
        









class LaxWendroffOperator:
    
    velocity = 0.
    dx       = .1
    dt       = .1
    n        = 10
    courant  = 0.
    
    
    def __init__(self, velocity, dx, dt, n):
        
        self.velocity = velocity
        self.dx       = dx
        self.dt       = dt
        self.n        = n
        
        self.courant = velocity*dt/dx
            

        if (velocity>0.):
            extrap = 'high'
        elif (velocity<0.):
            extrap = 'low'
        else:
            extrap = 'none'
        
        extrap = 'none'
        
        D1 = FirstDerivativeOperator( dx, n, extrap )
        D2 = SecondDerivativeOperator( dx, n, extrap )
        
        self.interior    = eye(n) - velocity*dt*D1.interior + .5*(velocity*dt)**2 * D2.interior


        if velocity > 0.:
            self.interior[-1,-3:] = array([0.,0.,1.]) - self.courant*array([.5, -2., 1.5]) + self.courant**2/2*array([1.,-2.,1.])
        elif velocity < 0.:
            self.interior[0,:3] = array([1.,0.,0.]) - self.courant*array([-1.5, 2., -.5]) + self.courant**2/2*array([1.,-2.,1.])


        self.oldBoundary = zeros( (n,2) )
        
        if velocity > 0.:
            self.oldBoundary[:,0] = -velocity*dt * D1.boundary[:,0] + .5*(velocity*dt)**2 * D2.boundary[:,0]
        elif velocity < 0.:
            self.oldBoundary[:,1] = -velocity*dt * D1.boundary[:,1] + .5*(velocity*dt)**2 * D2.boundary[:,1]
        
        # self.oldBoundary = -velocity*dt * D1.boundary + .5*(velocity*dt)**2 * D2.boundary
        
        self.newBoundary = zeros( (n,2) )
        
        
    def __call__( self, q, bc_old ):
        
        return dot( self.interior, q ) + dot( self.oldBoundary, bc_old )




class ForwardEulerOperator:
    
    diffusivity = 0.
    dx = .1
    dt = 0.
    n  = 10
    
    
    def __init__(self, diffusivity, dx, dt, n):
        
        self.diffusivity = diffusivity
        self.dx          = dx
        self.dt          = dt
        self.n           = n
        

        D2 = SecondDerivativeOperator(self.dx, self.n)
        
        self.interior = eye(n) + diffusivity*dt * D2.interior
        self.oldBoundary = diffusivity * dt * D2.boundary        
        self.newBoundary = zeros( (n,2) )
        

    def __call__( self, q, bc_old ):

        return dot( self.interior, q ) + dot( self.oldBoundary, bc_old )




class BackwardEulerOperator:
    
    diffusivity = 0.
    dx = .1
    dt = 0.
    n  = 10
    
    
    def __init__(self, diffusivity, dx, dt, n):
        
        self.diffusivity = diffusivity
        self.dx          = dx
        self.dt          = dt
        self.n           = n


        D2 = SecondDerivativeOperator( dx, n )
        
        self.interior = linalg.inv( eye(n) - diffusivity*dt * D2.interior )
        
        self.oldBoundary = zeros( (n,2) )
                
        self.newBoundary = diffusivity*dt * dot( self.interior,  D2.boundary )
        

    def __call__( self, q, bc_new ):

        return dot( self.interior, q ) + dot( self.newBoundary, bc_new )



class CrankNicolsonOperator:
    
    diffusivity = 0.
    dx = .1
    dt = 0.
    n  = 10
    
    
    def __init__(self, diffusivity, dx, dt, n):
        
        self.diffusivity = diffusivity
        self.dx          = dx
        self.dt          = dt
        self.n           = n


        FE = ForwardEulerOperator( self.diffusivity, self.dx, self.dt/2, self.n )
        BE = BackwardEulerOperator( self.diffusivity, self.dx, self.dt/2, self.n )
        
        self.interior    = dot( BE.interior, FE.interior )

        self.oldBoundary = dot( BE.interior, FE.oldBoundary )
        
        self.newBoundary = BE.newBoundary

    def __call__( self, q, bc_old, bc_new ):
        
        return dot( self.interior, q ) + dot( self.oldBoundary, bc_old ) + dot( self.newBoundary, bc_new )











        
# from pylab import *
# import time
# 
# 
# class AdvectingGaussian:
#     velocity = 1.
# 
#     def __init__(self, velocity):
#         self.velocity = velocity
# 
#     def __call__(self, x, t):
#         return exp(-200*(x-velocity*t)**2)
# 
# 
# 
# x_low  = 0.0
# x_high = 0.7
# 
# x = linspace(x_low, x_high, 101)
# x_interior = x[1:-1]
# x_boundary = x[[0,-1]]
# 
# dx = x[1] - x[0]
# n = len(x_interior)
# t_final = 1.3
# dt = .01
# velocity = 0.5
# 
# 
# q_true = AdvectingGaussian(velocity)
# q = q_true(x_interior,0.)
# 
# clf()
# p, = plot(x_interior, q, 'ro')
# p_true, = plot( x_interior, q_true(x_interior, 0.))
# time.sleep(.1)
# 
# 
# 
# LW = LaxWendroffOperator( velocity, dx, dt, n )
# 
# 
# t = 0.
# 
# while (t < t_final-1e-12):
#     
#     bc_old = q_true( x_boundary, t )
#     bc_new = q_true( x_boundary, t+dt )
#     
#     q = LW( q, bc_old )
#     
#     
#     t += dt
#     
#     p.set_ydata(q)
#     p_true.set_ydata( q_true(x_interior,t) )
#     draw()
#     time.sleep(.05)












# class SineMode:
#     diffusivity = 0.
#     mode = 1
# 
#     def __init__(self, diffusivity, mode):
#         self.diffusivity = diffusivity
#         self.mode = mode
# 
#     def __call__(self, x, t):
#         return exp(-(self.mode*pi)**2 * self.diffusivity * t) * sin(self.mode*pi*x)
# 
# 
# 
# x = linspace(-.8, .7, 101)
# x_interior = x[1:-1]
# x_boundary = x[[0,-1]]
# 
# dx = x[1] - x[0]
# n = len(x_interior)
# t_final = .4
# dt = .01
# diffusivity = .1
# 
# 
# q_true = SineMode(diffusivity, 2)
# q = q_true(x_interior,0.)
# time.sleep(.2)
# 
# clf()
# p, = plot(x_interior, q, 'ro')
# p_true, = plot( x_interior, q_true(x_interior, 0.))
# 
# 
# 
# BE = BackwardEulerOperator( diffusivity, dx, dt, n )
# CN = CrankNicolsonOperator( diffusivity, dx, dt, n )
# 
# Op = BE
# 
# 
# t = 0.
# 
# while (t < t_final-1e-12):
# 
#     bc_old = q_true( x_boundary, t )
#     bc_new = q_true( x_boundary, t+dt )
# 
#     q = CN( q, bc_old, bc_new )
#     
# 
#     t += dt
# 
#     p.set_ydata(q)
#     p_true.set_ydata( q_true(x_interior,t) )
#     draw()
#     time.sleep(.1)