from numpy import *
from IntegrationOperators import *
from BCEstimators import *



class StabilityTester:

    def __init__( self, mx_values, stab_values ):

        M, S = meshgrid(mx_values, stab_values)
        max_eigenvalues = zeros( shape(M) )

        for i in range( shape(M)[0] ):
            for j in range( shape(M)[1] ):

                m = M[i,j]
                s = S[i,j]

                eigenvalues, eigenvectors = linalg.eig( self.matrix(m,s) )

                max_eigenvalues[i,j] = abs(eigenvalues).max()
                
                print "Finished calculation for indices ", (i,j)

        self.max_eigenvalues = max_eigenvalues
        


class LaxWendroffForwardEst1(StabilityTester):
    
    def matrix(self, mx, courant):
        
        velocity = 1.
        dx = .1
        dt = courant * dx / velocity
        
        LW     = LaxWendroffOperator( velocity, dx, dt, mx )
        FwdEst = ForwardAdvectionBCEstimator( velocity, dx, dt, mx, order=1 )
        
        return dot( LW.interior, LW.interior ) + dot( LW.oldBoundary, FwdEst.interior )



class LaxWendroffForwardEst2(StabilityTester):

    def matrix(self, mx, courant):

        velocity = 1.
        dx = .1
        dt = courant * dx / velocity

        LW     = LaxWendroffOperator( velocity, dx, dt, mx )
        FwdEst = ForwardAdvectionBCEstimator( velocity, dx, dt, mx, order=2 )

        return dot( LW.interior, LW.interior ) + dot( LW.oldBoundary, FwdEst.interior )



class LaxWendroffBackwardEstimate(StabilityTester):
    
    def matrix(self, mx, courant):
        
        velocity = 1.
        dx = .1
        dt = courant * dx / velocity
        
        LW      = LaxWendroffOperator( velocity, dx, dt, mx )
        BackEst = BackwardAdvectionBCEstimator( velocity, dx, dt, mx )
        
        # return dot( LW.interior + dot(LW.oldBoundary,BackEst.interior), LW.interior )
        return LW.interior + dot( LW.oldBoundary, BackEst.interior )



class LaxWendroffStabilizedEst(StabilityTester):
    
    def matrix(self, mx, courant):

        velocity = 1.
        dx = .1
        dt = courant * dx / velocity
        
        LW      = LaxWendroffOperator( velocity, dx, dt, mx )
        StabEst = StabilizedAdvectionBCEstimator( velocity, dx, dt/2, dt, mx )

        return dot( LW.interior + dot(LW.oldBoundary,StabEst.interior), LW.interior )



class LaxWendroffUnmodified(StabilityTester):
    
    def matrix(self, mx, courant):

        velocity = 1.
        dx = .05
        dt = courant * dx / velocity
        
        LW = LaxWendroffOperator( velocity, dx, dt, mx )

        return LW.interior
    
    
        mx_values      = array( logspace(1,2,31).round(), dtype='int' )
        courant_values = linspace(0.5, 1.5, 31)
        M, C = meshgrid( mx_values, courant_values )

        LW = LaxWendroffUnmodified( mx_values, courant_values )

        # LWStabEst = LaxWendroffStabilizedEst( mx_values, courant_values )

        LWFwdEst = LaxWendroffForwardEst( mx_values, courant_values )
        
        
        

class CrankNicolsonStabilityTester(StabilityTester):
    
    def matrix( self, mx, stab_rat ):
        
        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity
        
        CN = CrankNicolsonOperator( diffusivity, dx, dt, mx )

        return CN.interior


class CrankNicolsonForwardStabilityTester(StabilityTester):
    
    def matrix( self, mx, stab_rat ):
        
        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity
        
        CN     = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        FwdEst = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )
        
        M1 = CN.interior + dot( CN.newBoundary, FwdEst.interior )
        
        return dot( CN.interior, M1 ) + dot( CN.oldBoundary, FwdEst.interior )
        

class CrankNicolsonOneStageForward(StabilityTester):
    
    def matrix( self, mx, stab_rat ):

        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity
        
        CN     = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        FwdEst = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )
                
        return CN.interior + dot( CN.newBoundary, FwdEst.interior )
        


class CrankNicolsonTwoStageForwardBackward(StabilityTester):
        
    def matrix( self, mx, stab_rat ):

        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity
        
        CN      = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        FwdEst  = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )      
        BackEst = BackwardDiffusionBCEstimator( diffusivity, dx, dt, mx )  
        
        M1 = CN.interior + dot( CN.newBoundary, FwdEst.interior )
        M2 = CN.interior + dot( CN.oldBoundary, BackEst.interior )
        
        return dot( M2, M1 )
        

class CrankNicolsonMiddleStage(StabilityTester):

    def matrix( self, mx, stab_rat ):

        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity
        
        CN      = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        FwdEst  = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )      
        BackEst = BackwardDiffusionBCEstimator( diffusivity, dx, 10*dt, mx )  

        M1 = FwdEst.interior + dot( FwdEst.oldBoundary, BackEst.interior )
        
        return CN.interior + dot( CN.oldBoundary, BackEst.interior ) + dot( CN.newBoundary, M1 )



class CrankNicolsonOneStageStabilized(StabilityTester):

    def matrix( self, mx, stab_rat ):

        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity

        CN      = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        FwdEst  = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )      
        BackEst = BackwardDiffusionBCEstimator( diffusivity, dx, 10*dt, mx )  

        M1 = FwdEst.interior + dot( FwdEst.oldBoundary, BackEst.interior )

        return CN.interior + dot( CN.newBoundary, M1 )

        
        
        
class CrankNicolsonBackwardStabilityTester(StabilityTester):

    def matrix( self, mx, stab_rat ):
        
        diffusivity = 1.
        dx = .1
        dt = stab_rat * dx**2 / diffusivity
        
        CN      = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        # FwdEst  = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )
        BackEst = BackwardDiffusionBCEstimator( diffusivity, dx, dt, mx )
        
        # M1 = CN.interior + dot( CN.newBoundary, FwdEst.interior )
        # M2 = CN.interior + dot( CN.oldBoundary, BackEst.interior )
        
        # return dot( M2, M1 )
        return CN.interior + dot( CN.oldBoundary, BackEst.interior )
        
        
        

class LWAdvDiffStabilityTester(StabilityTester):
    
    def matrix( self, courant, diff_ratio ):
    
        mx = 100
        dx = .1
        dt = .1
        
        velocity     = courant*dx/dt
        diffusivity  = diff_ratio*dx**2/dt
        
        LW      = LaxWendroffOperator( velocity, dx, dt, mx )
        BackEst = BackwardAdvDiffUpstreamBCEstimator( velocity, diffusivity, dx, dt, mx )
        
        return LW.interior + dot( LW.oldBoundary, BackEst.interior )
        
        


class CNAdvDiffStabilityTester(StabilityTester):
    
    def matrix( self, courant, diff_ratio ):
        
        mx = 100
        dx = .1
        dt = .1
        
        velocity     = courant*dx/dt
        diffusivity  = diff_ratio*dx**2/dt
        
        CN      = CrankNicolsonOperator( diffusivity/2, dx, dt, mx )
        BackEst = BackwardAdvDiffBCEstimator( velocity, diffusivity, dx, dt, mx )
        FwdEst  = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )
        
        M1 = FwdEst.interior + dot( FwdEst.oldBoundary, BackEst.interior )
        
        return CN.interior + dot( CN.oldBoundary, BackEst.interior ) + dot( CN.newBoundary, M1 )
        



        
class CNAdvDiffStabilityTester2(StabilityTester):

    def matrix( self, mx, diff_ratio ):

        courant = .9
        dx = .1
        dt = .1

        velocity     = courant*dx/dt
        diffusivity  = diff_ratio*dx**2/dt

        CN      = CrankNicolsonOperator( diffusivity, dx, dt, mx )
        BackEst = BackwardAdvDiffBCEstimator( velocity, diffusivity, dx, dt, mx )
        FwdEst  = ForwardDiffusionBCEstimator( diffusivity, dx, dt, mx )

        M1 = FwdEst.interior + dot( FwdEst.oldBoundary, BackEst.interior )

        return CN.interior + dot( CN.oldBoundary, BackEst.interior ) + dot( CN.newBoundary, M1 )