
from setrun import setrun
from setplot import setplot
from pyclaw.runclaw import runclaw
from pyclaw.plotters.plotclaw import plotclaw

from numpy import *
from pylab import *
import os




#---- dt refinement parameters ----
t_final_target = 1.0



    



def timeRefinementTest( diffusivity, dt_values, split_type ):
    
    n_runs = len(dt_values)
    
    rundata = setrun()
    rundata.clawdata.nout = 1
    rundata.clawdata.dt_variable = 0
    
    #---- Baseline discretization parameters ----
    rundata.clawdata.xlower =  1.
    rundata.clawdata.xupper =  2.


    #---- Set problem parameters ----
    rundata.probdata.diffusivity = diffusivity
    rundata.probdata.split_type  = split_type
    

    def velocityFunction(x):
        return (diffusivity*(4.*x**2 - 2.) + 1.) / (-2*x)
        
    x_sample = linspace(rundata.clawdata.xlower, rundata.clawdata.xupper, 101)
    max_velocity = abs( velocityFunction(x_sample) ).max()

    
    
    mx_base = 0
    
    
    dx_base = max_velocity*dt_values[0] / .95
    mx_base = int( (rundata.clawdata.xupper - rundata.clawdata.xlower) / dx_base )

        
    mx_values = zeros(n_runs, dtype='int')
    dx_values = zeros(n_runs)
    for i in range(n_runs):
        mx_values[i] = int(mx_base * dt_values[0] / dt_values[i])
        dx_values[i] = (rundata.clawdata.xupper - rundata.clawdata.xlower) / float(mx_values[i])
    print mx_values





    #===> Compute error for each dt ===>
    error_values   = zeros(n_runs)

    for i_run in range(n_runs):

        #---- Discretization parameters for this run ----
        dt      = dt_values[i_run]
        t_final = round(t_final_target/dt)*dt
        mx      = mx_values[i_run]
        
        #---- Write parameters to clawdata ----
        rundata.clawdata.dt_initial = dt
        rundata.clawdata.tfinal     = t_final
        rundata.clawdata.mx         = mx
    
        rundata.write()

        #---- Execute the run ----
        runclaw(xclawcmd = "xclaw", outdir="_output")
    
        #---- Read the final numerical error ----
        f = open("_output/error_value.txt", "r")
        error_values[i_run] = float(f.readline())
        f.close()
    
    #<=== Compute error for each dt <===
    
    return error_values, dx_values




dt_values = logspace(-1,-3,21)

lw_cn_lw_errors, dx_values = timeRefinementTest( 1e-1, dt_values, 1 )
cn_lw_cn_errors, dx_values = timeRefinementTest( 1e-1, dt_values, 2 )




#===> Produce error plots ===>

plot_parameters = {'backend': 'eps',
          'lines.linewidth': 4,
          'lines.markersize': 6,
          'text.fontsize': 10,
          'font.size'    : 10,
          'legend.fontsize': 10,
          'xtick.labelsize': 10,
          'ytick.labelsize': 10,
          'axes.labelsize':  14,
          'figure.subplot.left':   0.18,
          'figure.subplot.right':  0.95,
          'figure.subplot.bottom': 0.15,
          'figure.subplot.top':    0.9}
          
rcParams.update(plot_parameters)


figure(figsize=(4,3), facecolor='w', edgecolor='k')


# loglog(dt_values, dt_values,       'k--', label="$\Delta t$")
loglog(dt_values, dt_values**2,    'k-',  label="$\Delta t^2$")
loglog(dt_values, lw_cn_lw_errors, 'kx',  label="LW/CN/LW errors")
loglog(dt_values, cn_lw_cn_errors, 'k+',  label="CN/LW/CN errors")

# loglog(dt_values, cn_lw_errors, 'k+', label="CN/LW errors")


xlabel('$\Delta t$')

xlim(1e-3/1.3,2e-1)
ylim(1e-10,1e-1)

legend(loc='lower right')


#<=== Produce error plots <===





