subroutine diffusion_forward_estimate ( new_bcs, q, old_bcs, diffusivity, dx, dt, mx )
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(mx), old_bcs(2), diffusivity, dx, dt
    double precision, intent(out) :: new_bcs(2)
    double precision :: stab_rat
    
    stab_rat = diffusivity*dt/dx**2
    
    new_bcs(1) = old_bcs(1) + stab_rat * (2*old_bcs(1) - 5*q(1) + 4*q(2) - q(3))
    new_bcs(2) = old_bcs(2) + stab_rat * (-q(mx-2) + 4*q(mx-1) - 5*q(mx) + 2*old_bcs(2))
    
end subroutine diffusion_forward_estimate



subroutine advdiff_backward_estimate ( old_bcs, q, new_bcs, vel_bcs, diffusivity, dx, dt, mx )
    implicit none
    integer, intent(in) :: mx
    double precision, intent(in) :: q(mx), new_bcs(2), vel_bcs(2), diffusivity, dx, dt
    double precision, intent(out) :: old_bcs(2)
    double precision :: courant, diff_ratio

    diff_ratio = diffusivity*dt/dx**2

    
    courant  = vel_bcs(1)*dt/dx
    
    old_bcs(1) = ( new_bcs(1) + courant*q(1)                 &
                   - diff_ratio*(-5*q(1) + 4*q(2) - q(3)) )  &
                 / ( 1.d0 + courant + 2*diff_ratio )
    
    
    courant = vel_bcs(2)*dt/dx
    
    old_bcs(2) = ( new_bcs(2) - courant*q(mx)                      &
                   - diff_ratio*(-q(mx-2) + 4*q(mx-1) - 5*q(mx)) ) &
                / ( 1.d0 - courant + 2*diff_ratio )
                               
end subroutine advdiff_backward_estimate
