subroutine cn_lw_cn_split ( q, final_bcs, velocity, diffusivity, dx, dt, mx )

    implicit none
    
    !==== Variables ====
    integer, intent(in) :: mx
    double precision, intent(in) :: final_bcs(2), velocity(0:mx+1), diffusivity, dx, dt
    
    double precision, intent(inout) :: q(0:mx+1)
    
    double precision :: prev_bcs(2), next_bcs(2), vel_bcs(2)
    
    
    
    !---- Velocity at low and high boundaries (midway to boundary point) ----

    vel_bcs(1) = (velocity(0) + velocity(1))     / 2.d0
    vel_bcs(2) = (velocity(mx) + velocity(mx+1)) / 2.d0
    
    
    
    !---- First stage: Half diffusion ----
        
    call advdiff_backward_estimate( prev_bcs, q(1:mx), final_bcs, vel_bcs, diffusivity, dx, dt, mx )
    call diffusion_forward_estimate( next_bcs, q(1:mx), prev_bcs, diffusivity/2.d0, dx, dt, mx )
    
    q(0)    = prev_bcs(1)
    q(mx+1) = prev_bcs(2)

    call crank_nicolson_1d( q(0:mx+1), next_bcs, diffusivity/2.d0, dx, dt, mx )



    !---- Second stage: Full advection ----

    call advdiff_backward_estimate( prev_bcs, q(1:mx), final_bcs, vel_bcs, diffusivity/2.d0, dx, dt, mx )

    ! Set only the inflow (right) boundary value
    q(mx+1) = prev_bcs(2)

    call lax_wendroff_transport_1d( q(0:mx+1), velocity, dx, dt, mx )    


    

    !---- Third stage: Half diffusion ----
    
    call advdiff_backward_estimate( prev_bcs, q(1:mx), final_bcs, vel_bcs*0.d0, diffusivity/2.d0, dx, dt, mx )

    q(0)    = prev_bcs(1)
    q(mx+1) = prev_bcs(2)
    
    call crank_nicolson_1d( q(0:mx+1), final_bcs, diffusivity/2.d0, dx, dt, mx )

    
    
end subroutine cn_lw_cn_split