subroutine lax_wendroff_transport_1d( q, u, dx, dt, mx )

!---------------------------------------------------
! Takes a time step of the transport equation
!       q_t + u(x)q_x = 0
! using Lax-Wendroff.  Assumes outflow at the left,
! which isn't exactly ideal, but it gets my figure
! done for now.
!---------------------------------------------------
    
    implicit none
    
    !==== Variables ====
    !---- Input ----
    integer, intent(in) :: mx
    double precision, intent(in) :: u(0:mx+1), dx, dt
    
    !---- Input/output ----
    double precision, intent(inout) :: q(0:mx+1)
    
    !---- Locals ----
    integer :: ix
    double precision :: q_new(mx), q_x, q_xx, u_x
    
    


    ! ix=1
    q_x  = (-1.5d0*q(1) + 2*q(2) - .5d0*q(3)) / dx
    q_xx = (q(1) - 2*q(2) + q(3)) / dx**2
    u_x  = (-1.5d0*u(1) + 2*u(2) - .5*u(3)) / dx
    q_new(1) = q(1) - dt*u(1)*q_x  + dt**2/2 * (u(1)*u_x*q_x + u(1)**2*q_xx)


    
    do ix = 2,mx
        
        q_x  = (q(ix+1) - q(ix-1)) / (2*dx)
        q_xx = (q(ix+1) - 2*q(ix) + q(ix-1)) / dx**2
        u_x  = (u(ix+1) - u(ix-1)) / (2*dx)
        
        q_new(ix) = q(ix) - dt*u(ix)*q_x  + dt**2/2 * (u(ix)*u_x*q_x + u(ix)**2*q_xx)

        
    end do
    
    
    
        
        
    q(1:mx) = q_new


    
end subroutine lax_wendroff_transport_1d