
from setrun import setrun
from setplot import setplot
from pyclaw.runclaw import runclaw
from pyclaw.plotters.plotclaw import plotclaw

from numpy import *
import pylab, os



def timeRefinementTest(diffusivity, dt_values):
    
    n_runs = len(dt_values)
    
    rundata = setrun()
    rundata.clawdata.nout = 1
    rundata.clawdata.dt_variable = 0


    #---- Set problem parameters ----
    rundata.probdata.diffusivity = diffusivity


    #---- Baseline discretization parameters ----
    rundata.clawdata.xlower =  -2.
    rundata.clawdata.xupper =  -1.



    #---- dt refinement parameters ----
    t_final_target = 0.5


    
    
    mx_base = 0
    

    max_velocity = .5
    
    dx_base = max_velocity*dt_values[0] / 0.95
    mx_base = int( (rundata.clawdata.xupper - rundata.clawdata.xlower) / dx_base )

        
    mx_values = zeros(n_runs, dtype='int')
    for i in range(n_runs):
        mx_values[i] = int(mx_base * dt_values[0] / dt_values[i])
    print mx_values





    #===> Compute error for each dt ===>
    error_values   = zeros(n_runs)

    for i_run in range(n_runs):

        #---- Discretization parameters for this run ----
        dt      = dt_values[i_run]
        t_final = round(t_final_target/dt)*dt
        mx      = mx_values[i_run]
        
        #---- Write parameters to clawdata ----
        rundata.clawdata.dt_initial = dt
        rundata.clawdata.tfinal     = t_final
        rundata.clawdata.mx         = mx
    
        rundata.write()

        #---- Execute the run ----
        runclaw(xclawcmd = "xclaw", outdir="_output")
        plotclaw(outdir="_output", plotdir="_plots")
    
        #---- Read the final numerical error ----
        f = open("_output/error_value.txt", "r")
        error_values[i_run] = float(f.readline())
        f.close()
    
    #<=== Compute error for each dt <===
    
    return error_values




dt_values = logspace(-1,-3,21)

error_values = timeRefinementTest(0.0, dt_values)


#===> Produce error plots ===>

# plot_parameters = {'backend': 'eps',
#                    'lines.linewidth': 2,
#                    'lines.markersize': 4,
#                    'text.fontsize': 8,
#                    'legend.fontsize': 8,
#                    'xtick.labelsize': 8,
#                    'ytick.labelsize': 8,
#                    'axes.labelsize':  10,
#                    'figure.subplot.left':   0.2,
#                    'figure.subplot.right':  0.95,
#                    'figure.subplot.bottom': 0.2,
#                    'figure.subplot.top':    0.95}
#           
# pylab.rcParams.update(plot_parameters)


# fig = pylab.figure(figsize=(3,2), facecolor='w', edgecolor='k')
fig = pylab.figure()

ax = fig.add_subplot(1,1,1)

dt_squared = dt_values**2
ax.loglog(dt_values, dt_squared, 'k-', label="$\Delta t^2$ (advection)")
ax.loglog(dt_values, error_values, 'kx', label="Error values")


ax.set_xlabel('$\Delta t$')
ax.set_ylabel('Error')

# ax.set_xlim(1e-3/1.3,2e-1)
# ax.set_ylim(1e-9,3e-2)

ax.legend(loc='lower right')

# fig.savefig('Order1BCs.eps')
# os.system('epstopdf Order1BCs.eps')

#<=== Produce error plots <===





